'
'ISA Install
'Script for adding ISAPI filter in ISA Server/Forefront TMG
'

Option Explicit

'-----------------------------------------------------------------
'                               Host
'-----------------------------------------------------------------

Dim HostName
Dim HostNameDefault

'-----------------------------------------------------------------
'                              Filter
'-----------------------------------------------------------------

Dim FilterName
Dim FilterPath
Dim FilterPathDefault
Dim FilterDLLName
Dim FilterDesc

'-----------------------------------------------------------------
'                                ISA
'-----------------------------------------------------------------

Dim FilterGUID 						'for ISA install
Dim FilterRelativePath				'for ISA install
Dim FilterVendor					'for ISA install
Dim FilterVersion					'for ISA install

'-----------------------------------------------------------------
'                            Parameters
'-----------------------------------------------------------------

HostNameDefault = "LocalHost"
FilterPathDefault = "C:\Program Files\Microsoft ISA Server\AQTRONIX WebKnight\"
'FilterPathDefault = "C:\Program Files\Microsoft Forefront Threat Management Gateway\AQTRONIX WebKnight\"

FilterName = "WebKnight"
FilterDLLName = "WebKnight.dll"
FilterDesc = "AQTRONIX WebKnight - Web Application Firewall"

FilterGUID = "{6EF039AA-EE06-485a-86E0-E48262241B99}"
FilterRelativePath = "\AQTRONIX WebKnight\" 		'relative path for ISA install!!!
FilterVendor = "AQTRONIX"
FilterVersion = "4.2"

'uncomment the two lines below for unattended setup
HostName = HostNameDefault			    	'uncomment this for unattended setup
'FilterPath = FilterPathDefault				'uncomment this for unattended setup


InstallISA



'-----------------------------------------------------------------
'                           ISA Install
'-----------------------------------------------------------------

Sub InstallISA()
	'ask for hostname if we don't have one
	If HostName = "" Then
		HostName = AskHostName("Specify the ISA Server/Forefront TMG where you want to install " & FilterName)
	End If
	If HostName <> "" Then 'user did not cancel?
	
		If FilterPath = "" Then
			'try to detect path, if not found, ask path
			Dim ProgFiles
			ProgFiles = GetEnvironmentSetting("%PROGRAMFILES%") 'TODO: remote machine now has to use same path as local machine
			If ProgFiles = "" Then
				ProgFiles = "C:\Program Files" 'fail safe
			End If
			If FolderExists(FolderToUNC(HostName,ProgFiles & "\Microsoft Forefront Threat Management Gateway\")) Then
				'TMG "C:\Program Files\Microsoft Forefront Threat Management Gateway\AQTRONIX WebKnight\"
				FilterPath = ProgFiles & "\Microsoft Forefront Threat Management Gateway" & FilterRelativePath
			Else
				If FolderExists(FolderToUNC(HostName,ProgFiles & "\Microsoft ISA Server\")) Then
					'ISA "C:\Program Files\Microsoft ISA Server\AQTRONIX WebKnight\"
					FilterPath = ProgFiles & "\Microsoft ISA Server" & FilterRelativePath
				Else
					'don't know, ask the path with default to FilterPathDefault
					FilterPath = AskLocalPath
				End If
			End If
		End If
		
		If FilterPath <> "" Then 'user did not cancel?
			If Left(FilterPath,1)="\" Then	'no UNC paths!
				MsgBox "UNC paths not allowed!",,FilterName
			Else
				'add trailing slash if not there
				If Right(FilterPath,1)<>"\" And Right(FilterPath,1)<>"/" Then
					FilterPath = FilterPath & "\"
				End If

				'ISA root
				Dim FPC
				Set FPC = WScript.CreateObject("FPC.Root")
				'http://msdn.microsoft.com/en-us/library/aa503410.aspx

				'array
				Dim isaArray
				'ISA 2000
				'Set isaArray = FPC.Arrays.GetContainingArray
				'ISA 2000/2004/2006
				Set isaArray = FPC.Arrays(1)

				'list of installed filters
				Dim Filters
				Set Filters = isaArray.Extensions.WebFilters
	
				'copy files to host
				If CopyContent(HostName,FilterPath) Then

					'register filter in ISA
					Dim Filter
					Set Filter = Filters.Add(FilterGUID,FilterName, FilterRelativePath & FilterDLLName , 2, 2)   '2 == fpcFilterPriority_High
					'Filters.MoveDown  1
					Filter.Description = FilterDesc
					Filter.Vendor = FilterVendor
					Filter.Version = FilterVersion
					Filter.Enabled = True
					Filters.Save
					
					'notify user
					MsgBox FilterName & " is succesfully installed on " & HostName & vbCRLF & vbCRLF & "You should review the settings of " & FilterName & " (by running config.exe in " & FilterPath & ") and restart ISA Server/Forefront TMG on " & HostName & " to load the filter!",,FilterName

					'restart ISA: not necessary for web filter, restartmask = 0!
					'If MsgBox("Do you want to restart ISA Server/Forefront TMG now?",vbYesNo,FilterName)=vbYes Then
					'	Dim restartMask
					'	restartMask = filter.GetServiceRestartMask()
					'	isaArray.RestartServices restartMask
					'End If

					Set Filter = Nothing
				End If
				

				'cleanup
				Set Filters = Nothing
				Set isaArray = Nothing
				Set FPC = Nothing
			End If
		End If
	End If
End Sub

'-----------------------------------------------------------------
'                          ISA Uninstall
'-----------------------------------------------------------------

Sub UninstallISA()
	'ask for hostname if we don't have one
	If HostName = "" Then
		HostName = AskHostName("Specify the ISA Server/Forefront TMG from which you want to have " & FilterName & " uninstalled")
	End If
	If HostName <> "" Then 'user did not cancel?

		'ISA root
		Dim FPC
		Set FPC = WScript.CreateObject("FPC.Root")
		'http://msdn.microsoft.com/en-us/library/aa503410.aspx

		'array
		Dim isaArray
		'ISA 2000
		'Set isaArray = FPC.Arrays.GetContainingArray
		'ISA 2000/2004/2006
		Set isaArray = FPC.Arrays(1)

		'list of installed filters
		Dim Filters
		Set Filters = isaArray.Extensions.WebFilters
	
		'filter
		Dim Filter
		Set Filter = Filters(FilterGUID) 'this will result in an error if the filter is not found

		'get directory of installed filter
		FilterPath = ".." & Filter.RelativePath
		FilterPath = Left(FilterPath,Len(FilterPath)-Len(FilterDLLName))

		'remove trailing '\' (stupid DeleteFolder does not accept it)
		Do While Right(FilterPath,1)="\"
			FilterPath = Left(FilterPath,Len(FilterPath)-1)
		Loop
		
		'convert dir to unc path or leave it if it's local
		Dim uncpath
		uncpath = FolderToUNC(HostName,FilterPath)
		
		
		Filters.Remove(FilterGUID)
		Filters.Save

		'restart ISA: not necessary for web filter, restartmask = 0!
		'If MsgBox("Do you want to restart ISA Server/Forefront TMG now?",vbYesNo,FilterName)=vbYes Then
		'	Dim restartMask
		'	restartMask = filter.GetServiceRestartMask()
		'	isaArray.RestartServices restartMask
		'End If

		'remove installed dir if yes
		If MsgBox("Do you want to completely remove the directory " & uncPath & " as well?",vbYesNo,FilterName)=vbYes Then
			DeleteFolder uncpath
		End If	

		'notify user
		MsgBox FilterName & " was succesfully uninstalled.",,FilterName

		'cleanup
		Set Filter = Nothing
		Set Filters = Nothing
		Set isaArray = Nothing
		Set FPC = Nothing
	End If
End Sub

'-----------------------------------------------------------------
'                        File System
'-----------------------------------------------------------------

Function CopyContent(HostName,Path)
	'copy files to host
	Dim ret: ret = True
	Dim src
	Dim dest: dest = FolderToUNC(HostName,Path)
	Dim fso
	Set fso = CreateObject("Scripting.FileSystemObject")
	src = fso.GetAbsolutePathName(FilterDLLName)
	src = Left(src,Len(src)-Len(filterDLLName)) & "*.*"

	On Error Resume Next
	fso.CreateFolder dest
	Err.Clear
	fso.CopyFile src, dest, False
	If Err Then
		If MsgBox("Error copying the files to " & dest & ". If you know the error (access denied, default shares don't exist,...). You can continue setup and copy all the files manually. Click Yes to continue setup, No to cancel the installation." & vbCRLF & vbCRLF & "The specified error was: " & err.number & ": " & err.Description ,vbYesNo,FilterName)=vbNo Then
			ret = False
		End If
	End If
	On Error Goto 0

	CopyContent = ret
End Function

Sub DeleteFolder(path)
	Dim fso
	Set fso = CreateObject("Scripting.FileSystemObject")
	On Error Resume Next
	fso.DeleteFolder path, True
	if Err Then
		MsgBox "There was an error deleting the files. You should do it manually, setup will continue." & vbCRLF & vbCRLF & "The specified error is: " & err.Number & ": " & err.Description,,FilterName
	End If
	On Error Goto 0
	Set fso = Nothing
End Sub

Function FolderExists(path)
	Dim fso
	Set fso = CreateObject("Scripting.FileSystemObject")
	FolderExists = fso.FolderExists(path)
	Set fso = Nothing
End Function

Function FolderToUNC(HostName,Path)
	'convert dir to unc path or leave it if it's local
	If LCase(HostName)="localhost" Then
		FolderToUNC = Path
	Else
		FolderToUNC = "\\" & HostName & "\" & Replace(Path,":","$")
	End If
End Function

Sub SetNTFSPermission(Path,Username,Permission)
	'http://www.experts-exchange.com/Programming/Installation/InstallShield/Q_24315443.html
	If Right(Path,1) = "\" Then Path = Left(Path,Len(Path)-1)
	'MsgBox "cacls.exe """ & Path & """ /T /E /C /G """ & Username & """:" & Permission,,"Command"
	CreateObject("WScript.Shell").run "cacls.exe """ & Path & """ /T /E /C /G """ & Username & """:" & Permission
End Sub

Function GetEnvironmentSetting(Variable)
	On Error Resume Next
	Dim wshShell
	Set wshShell = CreateObject("WScript.Shell")
	GetEnvironmentSetting = wshShell.ExpandEnvironmentStrings(Variable)
	Set wshShell = Nothing
	On Error Goto 0
End Function

'-----------------------------------------------------------------
'                             Prompts
'-----------------------------------------------------------------

Function AskHostName(Message)
    'HostNameDefault has to be declared
    AskHostName = InputBox(Message,"Hostname required",HostNameDefault)
End Function

Function AskLocalPath()
    'HostName, FilterName, FilterPathDefault has to be declared
    AskLocalPath = InputBox("Specify a path on " & HostName & " where you want " & FilterName & " to be installed. This has to be an absolute path (with a local drive letter on that host, no UNC allowed here!)","Path required", FilterPathDefault)
End Function
'' SIG '' Begin signature block
'' SIG '' MIIdAwYJKoZIhvcNAQcCoIIc9DCCHPACAQExCzAJBgUr
'' SIG '' DgMCGgUAMGcGCisGAQQBgjcCAQSgWTBXMDIGCisGAQQB
'' SIG '' gjcCAR4wJAIBAQQQTvApFpkntU2P5azhDxfrqwIBAAIB
'' SIG '' AAIBAAIBAAIBADAhMAkGBSsOAwIaBQAEFP/jiT3SrbA0
'' SIG '' EdQrEEtzHDXdaj4+oIIYPDCCA+4wggNXoAMCAQICEH6T
'' SIG '' 6/t8xk5Z6kuad9QG/DswDQYJKoZIhvcNAQEFBQAwgYsx
'' SIG '' CzAJBgNVBAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENh
'' SIG '' cGUxFDASBgNVBAcTC0R1cmJhbnZpbGxlMQ8wDQYDVQQK
'' SIG '' EwZUaGF3dGUxHTAbBgNVBAsTFFRoYXd0ZSBDZXJ0aWZp
'' SIG '' Y2F0aW9uMR8wHQYDVQQDExZUaGF3dGUgVGltZXN0YW1w
'' SIG '' aW5nIENBMB4XDTEyMTIyMTAwMDAwMFoXDTIwMTIzMDIz
'' SIG '' NTk1OVowXjELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5
'' SIG '' bWFudGVjIENvcnBvcmF0aW9uMTAwLgYDVQQDEydTeW1h
'' SIG '' bnRlYyBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBIC0g
'' SIG '' RzIwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIB
'' SIG '' AQCxrLNJVEuXHBIK2CV5kSJXKm/cuCbEQ3Nrwr8uUFr7
'' SIG '' FMJ2jkMBJUO0oeJF9Oi3e8N0zCLXtJQAAvdN7b+0t0Qk
'' SIG '' a81fRTvRRM5DEnMXgotptCvLmR6schsmTXEfsTHd+1Fh
'' SIG '' AlOmqvVJLAV4RaUvic7nmef+jOJXPz3GktxK+Hsz5HkK
'' SIG '' +/B1iEGc/8UDUZmq12yfk2mHZSmDhcJgFMTIyTsU2sCB
'' SIG '' 8B8NdN6SIqvK9/t0fCfm90obf6fDni2uiuqm5qonFn1h
'' SIG '' 95hxEbziUKFL5V365Q6nLJ+qZSDT2JboyHylTkhE/xni
'' SIG '' RAeSC9dohIBdanhkRc1gRn5UwRN8xXnxycFxAgMBAAGj
'' SIG '' gfowgfcwHQYDVR0OBBYEFF+a9W5czMx0mtTdfe8/2+xM
'' SIG '' gC7dMDIGCCsGAQUFBwEBBCYwJDAiBggrBgEFBQcwAYYW
'' SIG '' aHR0cDovL29jc3AudGhhd3RlLmNvbTASBgNVHRMBAf8E
'' SIG '' CDAGAQH/AgEAMD8GA1UdHwQ4MDYwNKAyoDCGLmh0dHA6
'' SIG '' Ly9jcmwudGhhd3RlLmNvbS9UaGF3dGVUaW1lc3RhbXBp
'' SIG '' bmdDQS5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwDgYD
'' SIG '' VR0PAQH/BAQDAgEGMCgGA1UdEQQhMB+kHTAbMRkwFwYD
'' SIG '' VQQDExBUaW1lU3RhbXAtMjA0OC0xMA0GCSqGSIb3DQEB
'' SIG '' BQUAA4GBAAMJm495739ZMKrvaLX64wkdu0+CBl03X6ZS
'' SIG '' nxaN6hySCURu9W3rWHww6PlpjSNzCxJvR6muORH4KrGb
'' SIG '' sBrDjutZlgCtzgxNstAxpghcKnr84nodV0yoZRjpeUBi
'' SIG '' JZZux8c3aoMhCI5B6t3ZVz8dd0mHKhYGXqY4aiISo1EZ
'' SIG '' g362MIIEozCCA4ugAwIBAgIQDs/0OMj+vzVuBNhqmBsa
'' SIG '' UDANBgkqhkiG9w0BAQUFADBeMQswCQYDVQQGEwJVUzEd
'' SIG '' MBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xMDAu
'' SIG '' BgNVBAMTJ1N5bWFudGVjIFRpbWUgU3RhbXBpbmcgU2Vy
'' SIG '' dmljZXMgQ0EgLSBHMjAeFw0xMjEwMTgwMDAwMDBaFw0y
'' SIG '' MDEyMjkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMR0wGwYD
'' SIG '' VQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlvbjE0MDIGA1UE
'' SIG '' AxMrU3ltYW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNl
'' SIG '' cyBTaWduZXIgLSBHNDCCASIwDQYJKoZIhvcNAQEBBQAD
'' SIG '' ggEPADCCAQoCggEBAKJjCzlEuLsjp0RJuw7/ofBhClOT
'' SIG '' sJjbrSwPSsVu/4Y8U1UPFc4EPyv9qZaW2b5heQtbyUyG
'' SIG '' duXgQ0sile7CK0PBn9hotI5AT+6FOLkRxSPyZFjwFTJv
'' SIG '' TlehroikAtcqHs1L4d1j1ReJMluwXplaqJ0oUA4X7pbb
'' SIG '' YTtFUR3PElYLkkf8q672Zj1HrHBy55LnX80QucSDZJQZ
'' SIG '' vSWA4ejSIqXQugJ6oXeTW2XD7hd0vEGGKtwITIySjJEt
'' SIG '' nndEH2jWqHR32w5bMotWizO92WPISZ06xcXqMwvS8aMb
'' SIG '' 9Iu+2bNXizveBKd6IrIkri7HcMW+ToMmCPsLvalPmQjh
'' SIG '' EChyqs0CAwEAAaOCAVcwggFTMAwGA1UdEwEB/wQCMAAw
'' SIG '' FgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwDgYDVR0PAQH/
'' SIG '' BAQDAgeAMHMGCCsGAQUFBwEBBGcwZTAqBggrBgEFBQcw
'' SIG '' AYYeaHR0cDovL3RzLW9jc3Aud3Muc3ltYW50ZWMuY29t
'' SIG '' MDcGCCsGAQUFBzAChitodHRwOi8vdHMtYWlhLndzLnN5
'' SIG '' bWFudGVjLmNvbS90c3MtY2EtZzIuY2VyMDwGA1UdHwQ1
'' SIG '' MDMwMaAvoC2GK2h0dHA6Ly90cy1jcmwud3Muc3ltYW50
'' SIG '' ZWMuY29tL3Rzcy1jYS1nMi5jcmwwKAYDVR0RBCEwH6Qd
'' SIG '' MBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTIwHQYD
'' SIG '' VR0OBBYEFEbGaaMOShQe1UzaUmMXP142vA3mMB8GA1Ud
'' SIG '' IwQYMBaAFF+a9W5czMx0mtTdfe8/2+xMgC7dMA0GCSqG
'' SIG '' SIb3DQEBBQUAA4IBAQB4O7SRKgBM8I9iMDd4o4QnB28Y
'' SIG '' st4l3KDUlAOqhk4ln5pAAxzdzuN5yyFoBtq2MrRtv/Qs
'' SIG '' JmMz5ElkbQ3mw2cO9wWkNWx8iRbG6bLfsundIMZxD82V
'' SIG '' dNy2XN69Nx9DeOZ4tc0oBCCjqvFLxIgpkQ6A0RH83Vx2
'' SIG '' bk9eDkVGQW4NsOo4mrE62glxEPwcebSAe6xp9P2ctgwW
'' SIG '' K/F/Wwk9m1viFsoTgW0ALjgNqCmPLOGy9FqpAa8VnCwv
'' SIG '' SRvbIrvD/niUUcOGsYKIXfA9tFGheTMrLnu53CAJE3Hr
'' SIG '' ahlbz+ilMFcsiUk/uc9/yb8+ImhjU5q9aXSsxR08f5Lg
'' SIG '' w7wc2AR1MIIFDTCCA/WgAwIBAgIQAvUS3r+lbx/Gl9gq
'' SIG '' sKCoRDANBgkqhkiG9w0BAQsFADB2MQswCQYDVQQGEwJV
'' SIG '' UzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQL
'' SIG '' ExB3d3cuZGlnaWNlcnQuY29tMTUwMwYDVQQDEyxEaWdp
'' SIG '' Q2VydCBTSEEyIEhpZ2ggQXNzdXJhbmNlIENvZGUgU2ln
'' SIG '' bmluZyBDQTAeFw0xNTA1MjAwMDAwMDBaFw0xODA1MjQx
'' SIG '' MjAwMDBaMFYxCzAJBgNVBAYTAkJFMQ8wDQYDVQQHEwZC
'' SIG '' cnVnZ2UxGjAYBgNVBAoTEVBhcmNpZmFsIEFlcnRzc2Vu
'' SIG '' MRowGAYDVQQDExFQYXJjaWZhbCBBZXJ0c3NlbjCCASIw
'' SIG '' DQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKrRmpRy
'' SIG '' xaXsUiY6BweJp2DAS6SfPs5xIwrVvjg4wpZvbTBFOw0k
'' SIG '' XQHVaVflEdZ2Q+m19dDqgxWv5T/nqQrrg5u1LHZtp+Tx
'' SIG '' U2APC/Xu2g+pdfP+v0W3NlT+L7GZQMaOzWPDeuIOhh//
'' SIG '' YiDR6oeBCxXwP9EbUwHxt0yLsS8ZLXFCavnCfCVhI+WC
'' SIG '' wZgfZc+64MyF2YqTCwfNMSfR9B47RJnRY2xbilEMP7Mv
'' SIG '' 9L9ldLcH7SSntND08WpGUtQc7HzLebYf7UJbdvX+jHTu
'' SIG '' RJE918Ziow3piY50bLUCf6khAceObgnNHB3SENzsaWkp
'' SIG '' gM+fUI5Q86mY7pAhmH/6b7hITeMCAwEAAaOCAbUwggGx
'' SIG '' MB8GA1UdIwQYMBaAFGedDyAJDMyKOuWCRnJi/PHMkOVA
'' SIG '' MB0GA1UdDgQWBBR23QmHTOHgnyLvBRj8rEHHRDb9xjAO
'' SIG '' BgNVHQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUH
'' SIG '' AwMwbQYDVR0fBGYwZDAwoC6gLIYqaHR0cDovL2NybDMu
'' SIG '' ZGlnaWNlcnQuY29tL3NoYTItaGEtY3MtZzEuY3JsMDCg
'' SIG '' LqAshipodHRwOi8vY3JsNC5kaWdpY2VydC5jb20vc2hh
'' SIG '' Mi1oYS1jcy1nMS5jcmwwQgYDVR0gBDswOTA3BglghkgB
'' SIG '' hv1sAwEwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cu
'' SIG '' ZGlnaWNlcnQuY29tL0NQUzCBiAYIKwYBBQUHAQEEfDB6
'' SIG '' MCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2Vy
'' SIG '' dC5jb20wUgYIKwYBBQUHMAKGRmh0dHA6Ly9jYWNlcnRz
'' SIG '' LmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFNIQTJIaWdoQXNz
'' SIG '' dXJhbmNlQ29kZVNpZ25pbmdDQS5jcnQwDAYDVR0TAQH/
'' SIG '' BAIwADANBgkqhkiG9w0BAQsFAAOCAQEAd90BjUivgtU+
'' SIG '' uM7AaYBMDWB4udA9Ttrnlt/zx4Lb2wgoQcIA8NDRcxnY
'' SIG '' ocDn7gu160Nl/B2V0z6wyxeHklTSvqlI36zKs13Yfe5u
'' SIG '' M2FK+AtsKBcMjgvbGpzkSJMsCVSwDroEkwKvgtj8/GOe
'' SIG '' xalCzp8urZGbJpKAOGgNpGPgXbRrWiolCQta5PFrd5cz
'' SIG '' USMcqMIEplQUdovPPcDU826IcvocBka79Ry/H1jL6RVM
'' SIG '' jBfptdvFexzb5Su7Oj5jcXfti6WxM6ugh3EEvht/LF/q
'' SIG '' qevvKdh0MnltFLvNQ0U6ZZWhP6NRMTYD9tEAqtZ7ylty
'' SIG '' PBDaAOLAGKCagA94+UW8WjCCBTswggMjoAMCAQICCmEg
'' SIG '' TbQAAAAAACcwDQYJKoZIhvcNAQEFBQAwfzELMAkGA1UE
'' SIG '' BhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNV
'' SIG '' BAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBD
'' SIG '' b3Jwb3JhdGlvbjEpMCcGA1UEAxMgTWljcm9zb2Z0IENv
'' SIG '' ZGUgVmVyaWZpY2F0aW9uIFJvb3QwHhcNMTEwNDE1MTk0
'' SIG '' NTMzWhcNMjEwNDE1MTk1NTMzWjBsMQswCQYDVQQGEwJV
'' SIG '' UzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQL
'' SIG '' ExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdp
'' SIG '' Q2VydCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMIIB
'' SIG '' IjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAxszl
'' SIG '' c+b71LvlLS0ypt/lgT/JzSVJtnEqw9WUNGeiChywX2mm
'' SIG '' QLHEt7KP0JikqUFZOtPclNY823Q4pErMTSWC90qlUxI4
'' SIG '' 7vNJbXGRfmO2q6Zfw6SE+E9iUb74xezbOJLjBuUIkQzE
'' SIG '' KEFV+8taiRV+ceg1v01yCT2+OjhQW3cxG42zxyRFmqes
'' SIG '' bQAUWgS3uhPrUQqYQUEiTmVhh4FBUKZ5XIneGUpX1S7m
'' SIG '' XRxTLH6YzRoGFqRoc9A0BBNcoXHTWnxV215k4TeHMFYE
'' SIG '' 5RG0KYAS8Xk5iKICEXwnZreIt3jyygqoOKsKZMK/Zl2V
'' SIG '' hMGhJR6HXRpQCyASzEG7bgtROLhLywIDAQABo4HLMIHI
'' SIG '' MBEGA1UdIAQKMAgwBgYEVR0gADALBgNVHQ8EBAMCAYYw
'' SIG '' DwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQUsT7DaQP4
'' SIG '' v0cB1JgmGggC72NkK8MwHwYDVR0jBBgwFoAUYvsKIVt/
'' SIG '' Q24R2glUUGv10pZx8Z4wVQYDVR0fBE4wTDBKoEigRoZE
'' SIG '' aHR0cDovL2NybC5taWNyb3NvZnQuY29tL3BraS9jcmwv
'' SIG '' cHJvZHVjdHMvTWljcm9zb2Z0Q29kZVZlcmlmUm9vdC5j
'' SIG '' cmwwDQYJKoZIhvcNAQEFBQADggIBACCMwVntb5xrLcFK
'' SIG '' PnUdRUxBUBy9gOrZsJKLBioTP1MWnlY5aopjtngkefV9
'' SIG '' uLlHoQqWwvbLvaJmnwbhrNJ5CQ79PNysAgxwrz8b7Hh+
'' SIG '' 1OtLBWAm2XNhkSHtsGhj4JcSq2+gEu3Zn9LaJzyz5Fb5
'' SIG '' 0dSBD3G9QnymidzN1b2VoqvxkxF96KwxKahdZnBBnfx1
'' SIG '' ydWzGjkq0IUFUIuskcrEk8txpZ2klG9YDPpuIMQIMbWF
'' SIG '' nX6B+dI9ylsYhWwKhuwiCRuldDRPfyi8lUqrHbaYsF0J
'' SIG '' pHd2fu+njl2E9hgky9FtpsOhnMIQdYD/nTL95s9DOoL3
'' SIG '' zo/hciqbYrdf7ZUaOVwvlG1ItwFfMy+73C1zNIkEQgoc
'' SIG '' i3n5o/oX7/qhGhDf4LLBletcDAWXOzU+GIhN22y/JImN
'' SIG '' yL3Yn3s5OiSg1d/R80oal/amb3ofsJCps6wBOZHTYbdk
'' SIG '' 8T5XOAOvznrStZD1rtw5mdW2PJftpssWx31rKkyQlOZM
'' SIG '' VP0ezSDszmich1jpYWC+6w7J1Rl9n+l4vQ6sIXUHj6lu
'' SIG '' 4IxqKmuc4+dlvLwtPG3cBNxnRTYyrwSBvKgAbmFMlcVc
'' SIG '' 1I6Ony/BMnS9vRFlAwfN77deAlfahtQaKDSviEmyz6Xd
'' SIG '' glZvaKoU4llU/v/q7u/qknAiYIHjJSPAn8wPSbI1qljD
'' SIG '' OsPZFpQQMIIFTzCCBDegAwIBAgIQC34QkDw4SQ/6L2ea
'' SIG '' h6GnuTANBgkqhkiG9w0BAQsFADBsMQswCQYDVQQGEwJV
'' SIG '' UzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQL
'' SIG '' ExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdp
'' SIG '' Q2VydCBIaWdoIEFzc3VyYW5jZSBFViBSb290IENBMB4X
'' SIG '' DTEzMTAyMjEyMDAwMFoXDTI4MTAyMjEyMDAwMFowdjEL
'' SIG '' MAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IElu
'' SIG '' YzEZMBcGA1UECxMQd3d3LmRpZ2ljZXJ0LmNvbTE1MDMG
'' SIG '' A1UEAxMsRGlnaUNlcnQgU0hBMiBIaWdoIEFzc3VyYW5j
'' SIG '' ZSBDb2RlIFNpZ25pbmcgQ0EwggEiMA0GCSqGSIb3DQEB
'' SIG '' AQUAA4IBDwAwggEKAoIBAQC0Sl59Bw9B3sT1dhY2vXH/
'' SIG '' zz9Pc0uc0Q3+SstXWF6FFt0CFVSZ8I88L00CeBBoyNg1
'' SIG '' Sz/B92fOmByuM7ktHaQKVJPEhaLfNbH18TynszT7XUjJ
'' SIG '' RsliRLxImesoSVPDPY/ADt41mOliUd89a0Bh7gRB2s+n
'' SIG '' XFaW0flMt0SEh5hp5YK5E+ZVv8iScJIKMW9/izKrz2ta
'' SIG '' n2LEPu6+7VmkU38L8VKIinsKZyTLkM3s0k00TLDhtZ+c
'' SIG '' xvZvLM3mylN0AZ9nNd44SS3O7TlEghl5Thqytfu7ePBJ
'' SIG '' ZqfP+lyWdZKLGnLZ/1CSU8w+wkMyCRqGE2k8+4EyMzJk
'' SIG '' dXMoJh0IMDsHAgMBAAGjggHhMIIB3TASBgNVHRMBAf8E
'' SIG '' CDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNVHSUE
'' SIG '' DDAKBggrBgEFBQcDAzB/BggrBgEFBQcBAQRzMHEwJAYI
'' SIG '' KwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNv
'' SIG '' bTBJBggrBgEFBQcwAoY9aHR0cDovL2NhY2VydHMuZGln
'' SIG '' aWNlcnQuY29tL0RpZ2lDZXJ0SGlnaEFzc3VyYW5jZUVW
'' SIG '' Um9vdENBLmNydDCBjwYDVR0fBIGHMIGEMECgPqA8hjpo
'' SIG '' dHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRGlnaUNlcnRI
'' SIG '' aWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsMECgPqA8hjpo
'' SIG '' dHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRI
'' SIG '' aWdoQXNzdXJhbmNlRVZSb290Q0EuY3JsME8GA1UdIARI
'' SIG '' MEYwOAYKYIZIAYb9bAACBDAqMCgGCCsGAQUFBwIBFhxo
'' SIG '' dHRwczovL3d3dy5kaWdpY2VydC5jb20vQ1BTMAoGCGCG
'' SIG '' SAGG/WwDMB0GA1UdDgQWBBRnnQ8gCQzMijrlgkZyYvzx
'' SIG '' zJDlQDAfBgNVHSMEGDAWgBSxPsNpA/i/RwHUmCYaCALv
'' SIG '' Y2QrwzANBgkqhkiG9w0BAQsFAAOCAQEAag7/fhN8BqVL
'' SIG '' wC6M+VNkCeK6WJEwUOzMn+HTqC9IRjYYKdB4KF+YVkAP
'' SIG '' Hrq9sTuHXNxb2CAN7RoWTdURJCFL8SdpkBPrEaEB2v21
'' SIG '' TnlZdb04KmrD9o5BK4qii9csUVHZnKDI4066bKhH0k7R
'' SIG '' aB+MAlc7sylqjmogKrnyAGJkusjpAPnMpNS6mjXYryxl
'' SIG '' bBZ8WCHeSjDQ+uskXQbJnRa3rUpF0yXiDPBAqlxNrH7N
'' SIG '' BoK5dkZpCNgytoL+46lYNEMbjmdnlz9oMRY2OJU+h/fH
'' SIG '' w6+dencZ2d6Ttf1uK/yU+T23TBI1LDC+6I2eBXCaSBP0
'' SIG '' jNbnHqw456jzrQy3euxn7TGCBDMwggQvAgEBMIGKMHYx
'' SIG '' CzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJ
'' SIG '' bmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xNTAz
'' SIG '' BgNVBAMTLERpZ2lDZXJ0IFNIQTIgSGlnaCBBc3N1cmFu
'' SIG '' Y2UgQ29kZSBTaWduaW5nIENBAhAC9RLev6VvH8aX2Cqw
'' SIG '' oKhEMAkGBSsOAwIaBQCgcDAQBgorBgEEAYI3AgEMMQIw
'' SIG '' ADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgor
'' SIG '' BgEEAYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAjBgkqhkiG
'' SIG '' 9w0BCQQxFgQUrrhKE3SXJ94Eq6zfqAoTwrsHs+kwDQYJ
'' SIG '' KoZIhvcNAQEBBQAEggEATho6l6QTVgOr7cAL9YGb0MNI
'' SIG '' NNrthVPzEf3sJpquVECFHIZbSOJLMdLoj4Vargh1w1nl
'' SIG '' NvP2Mos6KK23A3C5zQzUzANX3HzA6lYR4gmK55RGmeI+
'' SIG '' xLIf8rmjyiHHJvFRrdYKdOV5OFXtKsj/28dCvzErMhJg
'' SIG '' c6y+lsfMyF3Uj6S4TSTUYougFp9BuSmlRkBvf6CKl8vk
'' SIG '' Vpr+NxR8G0z17rYjhXBlxj+q14/CjEMjDc+z2ydM6dpE
'' SIG '' HPZIvi7v8fMO9KWDjGOibnp3LZrH2EUzwuZbj0D7bDu2
'' SIG '' pSzDsBjC2f9dnskW2n9L7W6u2xuaZdrmxLhT9OHwr7KY
'' SIG '' 1370uTJT06GCAgswggIHBgkqhkiG9w0BCQYxggH4MIIB
'' SIG '' 9AIBATByMF4xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRT
'' SIG '' eW1hbnRlYyBDb3Jwb3JhdGlvbjEwMC4GA1UEAxMnU3lt
'' SIG '' YW50ZWMgVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQSAt
'' SIG '' IEcyAhAOz/Q4yP6/NW4E2GqYGxpQMAkGBSsOAwIaBQCg
'' SIG '' XTAYBgkqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqG
'' SIG '' SIb3DQEJBTEPFw0xNTA3MDExMTAzMTNaMCMGCSqGSIb3
'' SIG '' DQEJBDEWBBSxTnp7fBRtlTSAneiM19GNxMffjzANBgkq
'' SIG '' hkiG9w0BAQEFAASCAQAfro1nYpVqSlhW7TO1nI/uwlfW
'' SIG '' jH6OR7wIpm6bJ2I9qcHrnpOEC0o8LRsibnJfIBz4rcOG
'' SIG '' AAUYi4C31lY2VvUg8TR/3QDxo+jjBl9eCbdJh/Vb9Svr
'' SIG '' qnefwPk0KQGL5RmjtQc6IhdJVsubbIV7V0ukCpkNTLQk
'' SIG '' Y7hjYyCsAIQHqFaJvMZGTX0eclSxC7zY8u9uGUodM9XS
'' SIG '' YD3rIOHkTw/7Qo1ccv+Kibq7t1gkGe4DwQB/2dSO33sX
'' SIG '' /ecEWt/Qf6BqVRfyYUeElcXvNVwdFRUT7iWAg24OVsmQ
'' SIG '' m8zF/HM6vhoISAZg3hrGTG3yeNvqbpaO5/qmMAgajGXV
'' SIG '' dWV+rULQ
'' SIG '' End signature block
