/*
    AQTRONIX C++ Library
    Copyright 2003-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// HTML.cpp: implementation of the CHTML class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTML.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/* Changelog
 * 2006.04.02: Added HTMLToTags()
 *
 */

CHTML::CHTML()
{

}

CHTML::~CHTML()
{

}

CString CHTML::HTMLToText(CString &html)
{
	int length(html.GetLength());
	bool skip(false);
	CString text("");
	if(length<0)
		length=0;
	for(int i=0;i<length;i++){
		//start skipping htmltag
		if(html[i]=='<'){
			skip = true;
			if(i+2<length && (  (html[i+1]=='b' || html[i+1]=='B') &&
								(html[i+2]=='r' || html[i+2]=='R') &&
								(html[i+3]=='>')					  ) ){
				text += "\r\n";
			}
		}else{
			//copy everything else
			if(!skip){
				text += html[i];
			}
			//stop skipping htmltag
			if(html[i]=='>'){
				skip = false;
			}
		}
	}
	HTMLDecode(text);
	return text;
}

CString CHTML::HTMLToTags(CString &html)
{
	int length(html.GetLength());
	bool skip(true);
	CString text("");
	if(length<0)
		length=0;
	for(int i=0;i<length;i++){
		//start skipping htmltag
		if(html[i]=='<'){
			skip = false;
		}
		//copy everything else
		if(!skip){
			text += html[i];
		}
		//stop skipping htmltag
		if(html[i]=='>'){
			skip = true;
		}
	}
	return text;
}

CString& CHTML::HTMLEncode(CString& str)
{
	//standard encoded characters
	str.Replace("&","&amp;");
	str.Replace("<","&lt;");
	str.Replace(">","&gt;");
	str.Replace("'","&apos;");
	str.Replace("\"","&quot;");

	//TODO: HTML - more?
	
	return str;
}

CString& CHTML::HTMLDecode(CString& str)
{
	//standard encoded characters (ascii<127)
	str.Replace("&lt;","<");
	str.Replace("&gt;",">");
	str.Replace("&apos;","'");
	str.Replace("&quot;","\"");
	str.Replace("&frasl;","/");

	//standard encoded characters (ascii>127)
	str.Replace("&ndash;","-");		//en dash
	str.Replace("&mdash;","-");		//em dash
	str.Replace("&nbsp;"," ");		//nonbreaking space
	str.Replace("&iexcl;","");		//inverted exclamation
	str.Replace("&cent;","");		//cent sign
	str.Replace("&pound;","");		//pound sterling
	str.Replace("&curren;","");	//general currency sign
	str.Replace("&yen;","");		//yen sign
	str.Replace("&brvbar;","");	//broken vertical bar
	str.Replace("&brkbar;","");	//broken vertical bar
	str.Replace("&sect;","");		//section sign
	str.Replace("&uml;","");		//umlaut
	str.Replace("&die;","");		//umlaut
	str.Replace("&copy;","");		//copyright
	str.Replace("&ordf;","");		//feminine ordinal
	str.Replace("&laquo;","");		//left angle quote
	str.Replace("&not;","");		//not sign
	str.Replace("&shy;","");		//soft hyphen
	str.Replace("&reg;","");		//registered trademark
	str.Replace("&macr;","");		//macron accent
	str.Replace("&hibar;","");		//macron accent
	str.Replace("&deg;","");		//degree sign
	str.Replace("&plusmn;","");	//plus or minus
	str.Replace("&sup2;","");		//superscript two
	str.Replace("&sup3;","");		//superscript three
	str.Replace("&acute;","");		//acute accent
	str.Replace("&micro;","");		//micro sign
	str.Replace("&para;","");		//paragraph sign
	str.Replace("&middot;","");	//middle dot
	str.Replace("&cedil;","");		//cedilla
	str.Replace("&sup1;","");		//superscript one
	str.Replace("&ordm;","");		//masculine ordinal
	str.Replace("&raquo;","");		//right angle quote
	str.Replace("&frac14;","");	//one-fourth
	str.Replace("&frac12;","");	//one-half
	str.Replace("&frac34;","");	//three-fourths
	str.Replace("&iquest;","");	//inverted question mark
	str.Replace("&Agrave;","");	//uppercase A, grave accent
	str.Replace("&Aacute;","");	//uppercase A, acute accent
	str.Replace("&Acirc;","");		//uppercase A, circumflex accent
	str.Replace("&Atilde;","");	//uppercase A, tilde
	str.Replace("&Auml;","");		//uppercase A, umlaut
	str.Replace("&Aring;","");		//uppercase A, ring
	str.Replace("&AElig;","");		//uppercase AE
	str.Replace("&Ccedil;","");	//uppercase C, cedilla
	str.Replace("&Egrave;","");	//uppercase E, grave accent
	str.Replace("&Eacute;","");	//uppercase E, acute accent
	str.Replace("&Ecirc;","");		//uppercase E, circumflex accent
	str.Replace("&Euml;","");		//uppercase E, umlaut
	str.Replace("&Igrave;","");	//uppercase I, grave accent
	str.Replace("&Iacute;","");	//uppercase I, acute accent
	str.Replace("&Icirc;","");		//uppercase I, circumflex accent
	str.Replace("&Iuml;","");		//uppercase I, umlaut
	str.Replace("&ETH;","");		//uppercase Eth, Icelandic
	str.Replace("&Ntilde;","");	//uppercase N, tilde
	str.Replace("&Ograve;","");	//uppercase O, grave accent
	str.Replace("&Oacute;","");	//uppercase O, acute accent
	str.Replace("&Ocirc;","");		//uppercase O, circumflex accent
	str.Replace("&Otilde;","");	//uppercase O, tilde
	str.Replace("&Ouml;","");		//uppercase O, umlaut
	str.Replace("&times;","");		//multiplication sign
	str.Replace("&Oslash;","");	//uppercase O, slash
	str.Replace("&Ugrave;","");	//uppercase U, grave accent
	str.Replace("&Uacute;","");	//uppercase U, acute accent
	str.Replace("&Ucirc;","");		//uppercase U, circumflex accent
	str.Replace("&Uuml;","");		//uppercase U, umlaut
	str.Replace("&Yacute;","");	//uppercase Y, acute accent
	str.Replace("&THORN;","");		//uppercase THORN, Icelandic
	str.Replace("&szlig;","");		//lowercase sharps, German
	str.Replace("&agrave;","");	//lowercase a, grave accent
	str.Replace("&aacute;","");	//lowercase a, acute accent
	str.Replace("&acirc;","");		//lowercase a, circumflex accent
	str.Replace("&atilde;","");	//lowercase a, tilde
	str.Replace("&auml;","");		//lowercase a, umlaut
	str.Replace("&aring;","");		//lowercase a, ring
	str.Replace("&aelig;","");		//lowercase ae
	str.Replace("&ccedil;","");	//lowercase c, cedilla
	str.Replace("&egrave;","");	//lowercase e, grave accent
	str.Replace("&eacute;","");	//lowercase e, acute accent
	str.Replace("&ecirc;","");		//lowercase e, circumflex accent
	str.Replace("&euml;","");		//lowercase e, umlaut
	str.Replace("&igrave;","");	//lowercase i, grave accent
	str.Replace("&iacute;","");	//lowercase i, acute accent
	str.Replace("&icirc;","");		//lowercase i, circumflex accent
	str.Replace("&iuml;","");		//lowercase i, umlaut
	str.Replace("&eth;","");		//lowercase eth, Icelandic
	str.Replace("&ntilde;","");	//lowercase n, tilde
	str.Replace("&ograve;","");	//lowercase o, grave accent
	str.Replace("&oacute;","");	//lowercase o, acute accent
	str.Replace("&ocirc;","");		//lowercase o, circumflex accent
	str.Replace("&otilde;","");	//lowercase o, tilde
	str.Replace("&ouml;","");		//lowercase o, umlaut
	str.Replace("&divide;","");	//division sign
	str.Replace("&oslash;","");	//lowercase o, slash
	str.Replace("&ugrave;","");	//lowercase u, grave accent
	str.Replace("&uacute;","");	//lowercase u, acute accent
	str.Replace("&ucirc;","");		//lowercase u, circumflex accent
	str.Replace("&uuml;","");		//lowercase u, umlaut
	str.Replace("&yacute;","");	//lowercase y, acute accent
	str.Replace("&thorn;","");		//lowercase thorn, Icelandic
	str.Replace("&yuml;","");		//lowercase y, umlaut

	//Decode characters that cannot be represented
	//in pure ascii text
	str.Replace("&lsquo;","");		//left single quote
	str.Replace("&rsquo;","");		//right single quote
	str.Replace("&sbquo;","?");		//single low-9 quote
	str.Replace("&ldquo;","?");		//left double quote
	str.Replace("&rdquo;","?");		//right double quote
	str.Replace("&bdquo;","?");		//double low-9 quote
	str.Replace("&dagger;","?");	//dagger
	str.Replace("&Dagger;","?");	//double dagger
	str.Replace("&permil;","?");	//per mill sign
	str.Replace("&lsaquo;","?");	//single left-pointing angle quote
	str.Replace("&rsaquo;","?");	//single right-pointing angle quote
	str.Replace("&spades;","?");	//black spade suit
	str.Replace("&clubs;","?");		//black club suit
	str.Replace("&hearts;","?");	//black heart suit
	str.Replace("&diams;","?");		//black diamond suit
	str.Replace("&oline;","?");		//overline, = spacing overscore
	str.Replace("&larr;","?");		//leftward arrow
	str.Replace("&uarr;","?");		//upward arrow
	str.Replace("&rarr;","?");		//rightward arrow
	str.Replace("&darr;","?");		//downward arrow
	str.Replace("&trade;","(TM)");	//trademark sign

	//Decode "&#xxx;" values
	CString tmp;
	char bufint[34] = "\0";
	char buf[2] = "\0";
	buf[1] = '\0';
	for(unsigned char c=1;c<255;c++){
		tmp = "&#";
		tmp += itoa((int)c,bufint,10);
		tmp += ';';
		buf[0] = c;
		str.Replace(tmp,buf);
	}

	//last
	str.Replace("&amp;","&");

	return str;
}

CString CHTML::FilterTag(CString &HTML, CString TagName)
{
	/*
	 * Removes a tag from the html code
	 */

	//remove a tag: the "<TagName - </TagName>" part
	int taglength = TagName.GetLength()+3;
	int posstart(0); int posstop(-taglength);
	CString html(HTML);
	CString ret;
	html.MakeLower();

	posstart = html.Find("<" + TagName);

	while(posstart!=-1 && posstop!=-1 && posstop<posstart){
		ret += HTML.Mid(posstop+taglength,posstart-posstop-taglength);
		posstop = html.Find("</" + TagName + ">",__max(posstop+taglength,posstart));
		posstart = html.Find("<" + TagName,__max(posstart,posstop+taglength));
	}

	//rest of html
	if(posstop!=-1){
		ret += HTML.Mid(posstop+taglength);
	}
	
	return ret;
}



CString CHTML::ExtractTag(CString &HTML, CString TagName, int Start)
{
	/* 
	 * Extracts a tag at a certain position from first tag in the HTML
	 */
	//extract the "<tagname - </tagname>" part
	CString Tag(HTML);
	Tag.MakeLower();
	//find first tag in html, this is position zero!
	int posstart; posstart = Tag.Find("<" + TagName);
	if(posstart==-1)
		return "";
	//if start is not 0, add Start to posstart
	if(Start!=0){
		posstart+=Start;
		posstart = Tag.Find("<" + TagName,posstart);
		if(posstart==-1)
			return "";
	}
	//stop
	int posstop; posstop = Tag.Find("</" + TagName + ">",posstart);
	if(posstop==-1)
		posstop=HTML.GetLength()-1;
	//if ok, extract...
	if(posstart<posstop){
		Tag = HTML.Mid(posstart,posstop-posstart+TagName.GetLength()+3);
	}else{
		Tag = "";
	}
	return Tag;
}

CString CHTML::ExtractAllTags(CString &HTML, CString TagName)
{
	/*
	 * Extracts all tags from hmtl
	 */
	int pos(0);
	CString extract("");
	CString ret("");
	do{
		extract = ExtractTag(HTML,TagName,pos);
		ret += extract;
		pos += extract.GetLength();
	}while(extract!="");
	
	return ret;
}


