/*
    AQTRONIX C++ Library
    Copyright 2004-2006 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// HTTPFirewallSettings.cpp: implementation of the CHTTPFirewallSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "HTTPFirewallSettings.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHTTPFirewallSettings::CHTTPFirewallSettings()
{
	LoadDefaults(); //always load defaults (failover redundancy)
}

CHTTPFirewallSettings::~CHTTPFirewallSettings()
{

}

void CHTTPFirewallSettings::LoadDefaults()
{
	Admin.ServerHeader = "CHTTPFirewall/3.0";
	HTTPResponse.Contents = "HTTP Firewall Alert";
}

bool CHTTPFirewallSettings::ProcessFile(LPCTSTR fn)
{
	//if file is a urlscan.ini file
	CFileName cfn(fn);
	if(cfn.FileNameLCase == "urlscan.ini"){		//urlscan.ini file
		return ReadFromFileURLScan(fn);
	}else if(cfn.ExtensionLCase == ".ini"){		//.ini file
		return ReadFromFileINI(fn);
	}else if(cfn.ExtensionLCase == ".xml"){		//.xml file
		return ReadFromFileXML(fn);
	}else{
		return false;							//unknown file type
	}
}

bool CHTTPFirewallSettings::ReadFromFileURLScan(LPCTSTR fn)
{
	/* Read settings from a urlscan.ini file
	 * All checks on the file (exists? access allowed? ...)
	 * should have been done before this function is called
	 */
	try{
		const int sz = SETTINGS_MAX_INI_BUF_SIZE;
		char buf[sz];

		//[options]
		Verbs.AllowVerbs.Action = GetPrivateProfileInt("options","UseAllowVerbs",Verbs.AllowVerbs.Action,fn);
		Verbs.DenyVerbs.Action = !GetPrivateProfileInt("options","UseAllowVerbs",!Verbs.DenyVerbs.Action,fn);
		Extensions.AllowExtensions.Action = GetPrivateProfileInt("options","UseAllowExtensions",Extensions.AllowExtensions.Action,fn);
		Extensions.DenyExtensions.Action = !GetPrivateProfileInt("options","UseAllowExtensions",!Extensions.DenyExtensions.Action,fn);
		Path.Dot = !GetPrivateProfileInt("options","AllowDotInPath",!Path.Dot,fn);
		Logging.Enabled = GetPrivateProfileInt("options","EnableLogging",Logging.Enabled,fn)>0;
		Engine.AllowLateScanning = GetPrivateProfileInt("options","AllowLateScanning",Engine.AllowLateScanning,fn)>0;
		
		CString url(HTTPResponse.RedirectURL);
		GetPrivateProfileString("options","RejectResponseUrl","",buf,sz,fn);
		HTTPResponse.RedirectURL = buf; CleanupString(HTTPResponse.RedirectURL);
		if(HTTPResponse.RedirectURL=="") HTTPResponse.RedirectURL=url;
		HTTPResponse.Redirect = !GetPrivateProfileInt("options","UseFastPathReject",!HTTPResponse.Redirect,fn);
		HTTPResponse.Directly = GetPrivateProfileInt("options","UseFastPathReject",0,fn)>0;

		ResponseMonitor.RemoveServerHeader = GetPrivateProfileInt("options","RemoveServerHeader",ResponseMonitor.RemoveServerHeader,fn)>0;
		GetPrivateProfileString("options","AlternateServerName","",buf,sz,fn);
		ResponseMonitor.ServerHeader = buf; CleanupString(ResponseMonitor.ServerHeader);
		ResponseMonitor.ChangeServerHeader = ResponseMonitor.ServerHeader.GetLength()>0;

		CString ldir(Logging.LogDirectory);
		GetPrivateProfileString("options","LoggingDirectory","",buf,sz,fn);
		Logging.LogDirectory = buf; CleanupString(Logging.LogDirectory);
		if(Logging.LogDirectory=="") Logging.LogDirectory=ldir;

		//NormalizeURLBeforeScan = GetPrivateProfileInt("options","NormalizeURLBeforeScan",1,fn);
		//VerifyNormalization = GetPrivateProfileInt("options","VerifyNormalization",1,fn);
		URL.HighBitShellcode = !GetPrivateProfileInt("options","AllowHighBitCharacters",!URL.HighBitShellcode,fn);
	
		//LogLongUrls = GetPrivateProfileInt("options","LogLongUrls",0,fn);
		//PerDayLogging = GetPrivateProfileInt("options","PerDayLogging",1,fn);
		Logging.PerProcess = GetPrivateProfileInt("options","PerProcessLogging",0,fn)>0;
		
		DWORD l;

		//[AllowVerbs]
		l = GetPrivateProfileSection("AllowVerbs",buf, sz,fn);
		Split(buf,l,Verbs.AllowVerbs.List);
		CleanupList(Verbs.AllowVerbs.List);
		
		//[DenyVerbs]
		l = GetPrivateProfileSection("DenyVerbs",buf, sz,fn);
		Split(buf,l,Verbs.DenyVerbs.List);
		CleanupListUCase(Verbs.DenyVerbs.List);
		
		//[DenyHeaders]
		l = GetPrivateProfileSection("DenyHeaders",buf, sz,fn);
		Split(buf,l,Headers.DenyHeaders.List);
		CleanupList(Headers.DenyHeaders.List);
		Headers.DenyHeaders.Action = l>0;

		//[AllowExtensions]
		l = GetPrivateProfileSection("AllowExtensions",buf, sz,fn);
		Split(buf,l,Extensions.AllowExtensions.List);
		if(l) Extensions.AllowExtensions.List.AddHead("");			//always allow default document!!!
		CleanupListLCase(Extensions.AllowExtensions.List);

		//[DenyExtensions]
		l = GetPrivateProfileSection("DenyExtensions",buf, sz,fn);
		Split(buf,l,Extensions.DenyExtensions.List);
		CleanupListLCase(Extensions.DenyExtensions.List);
		
		//[DenyUrlSequences]
		l = GetPrivateProfileSection("DenyUrlSequences",buf, sz,fn);
		Split(buf,l,URL.DenySequences.List);
		CleanupListLCase(URL.DenySequences.List);
		URL.DenySequences.Action = l>0;

		//[AlwaysAllowedUrls]
		l = GetPrivateProfileSection("AlwaysAllowedUrls",buf, sz,fn);
		Split(buf,l,URL.ExcludedURL.List);
		CleanupList(URL.ExcludedURL.List);
		URL.ExcludedURL.Enabled = l>0;

		//[AlwaysAllowedQueryStrings]
		l = GetPrivateProfileSection("AlwaysAllowedQueryStrings",buf, sz,fn);
		Split(buf,l,QueryString.ExcludedQueryString.List);
		CleanupList(QueryString.ExcludedQueryString.List);
		QueryString.ExcludedQueryString.Enabled = l>0;

		//[RequestLimits]
		GetPrivateProfileString("RequestLimits","MaxAllowedContentLength",Limits.MaxContentLength.Value,buf,sz,fn);
		Limits.MaxContentLength.Value = buf; CleanupString(Limits.MaxContentLength.Value);
		Limits.MaxContentLength.Action = Limits.MaxContentLength.Value.GetLength()>0;

		URL.MaxLength.Value = GetPrivateProfileInt("RequestLimits","MaxUrl",URL.MaxLength.Value,fn);
		URL.MaxLength.Action = URL.MaxLength.Value>0;

		QueryString.MaxLength.Value = GetPrivateProfileInt("RequestLimits","MaxQueryString",QueryString.MaxLength.Value,fn);
		QueryString.MaxLength.Action = QueryString.MaxLength.Value>0;

		l = GetPrivateProfileSection("RequestLimits",buf, sz,fn);
		CStringList MaxLimits; POSITION oldpos;
		Split(buf,l,MaxLimits);
		POSITION pos = MaxLimits.GetHeadPosition();
		while (pos != NULL){
			oldpos = pos;
			CString& str = MaxLimits.GetNext(pos);
			CleanupString(str);
			if (str.Left(4)!="Max-")
				MaxLimits.RemoveAt(oldpos);
		}
		if(l>0){
			Headers.MaxHeaders.Map.RemoveAll();	//if section is found, remove defaults
			pos = MaxLimits.GetHeadPosition();
			CString val; int poseq;
			while (pos!=NULL){
				CString& str = MaxLimits.GetNext(pos);
				poseq = str.Find('=');
				if(poseq!=-1){
					if(str.Mid(poseq+1) != "")
						Headers.MaxHeaders.Map.SetAt(str.Mid(4,poseq-4)+":",str.Mid(poseq+1));
				}
			}
			Headers.MaxHeaders.Action = Action::Block;
		}else{
			Headers.MaxHeaders.Action = Action::Disabled;
		}

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::ReadFromActiveDirectory()
{
	/* Read settings from Active Directory (enterprise policy)
	 * If you want to apply the same settings for multiple
	 * machines, use this.
	 * Requires domain controller (LDAP server)
	 */
	try{
		//TODO: SETTINGS - ReadFromActiveDirectory
		
		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::ReadFromRegistry()
{
	/*
	 * read settings from the windows registry
	 */
	try{
		//TODO: SETTINGS - ReadFromRegistry
		//parser gebruiken?

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::WriteToRegistry()
{
	/*
	 * Write settings to the windows registry (machine policy)
	 */
	try{
		//TODO: SETTINGS - WriteToRegistry
		
		return true;
	}catch(...){
		return false;
	}
}

bool CHTTPFirewallSettings::WriteToActiveDirectory()
{
	/*
	 * Write settings to Active Directory (enterprise policy)
	 */
	try{
		//TODO: SETTINGS - WriteToActiveDirectory

		return true;
	}catch(...){
		return false;
	}
}

bool CHTTPFirewallSettings::ReadFromFileINI(LPCTSTR fn)
{
	try{
		const int sz = SETTINGS_MAX_INI_BUF_SIZE;
		char buf[sz];
		DWORD l;

		CFirewallSettings::ReadFromFileINI(fn);

		//Scanning Engine
		READ_INI_BOOL(Engine,AllowLateScanning)
		READ_INI_BOOL(Engine,ScanNonSecurePort)
		READ_INI_BOOL(Engine,ScanSecurePort)
		READ_INI_LIST(Engine,ExcludedInstances,CleanupList)
		READ_INI_LIST(Engine,ExcludedHosts,CleanupList)
		READ_INI_LIST(Engine,ExcludedIPAddresses,CleanupList)
				
		//Intrusion Response Handling
		_HTTPResponse& Response = HTTPResponse;
		READ_INI_BOOL(Response,Directly)
		READ_INI_BOOL(Response,Redirect)
		READ_INI_STRING(Response,RedirectURL)
		READ_INI_BOOL(Response,UseStatus)
		READ_INI_STRING(Response,Status)
		READ_INI_BOOL(Response,DropConnection)
		READ_INI_BOOL(Response,MonitorIP)
		READ_INI_INT(Response,MonitorIPTimeout)
		READ_INI_CACHE(Response,BlockIP)

		//Logging
		_HTTPLogging& Logging = HTTPLogging;
		READ_INI_BOOL(Logging,Log_HTTP_VIA)
		READ_INI_BOOL(Logging,Log_HTTP_X_FORWARDED_FOR)
		READ_INI_BOOL(Logging,LogHostHeader)
		READ_INI_BOOL(Logging,LogUserAgent)

		//Connection
		READ_INI_STRING(Connection,ClientIPVariable)
		READ_INI_BOOL(Connection,ChangeLogIPVariable)
		READ_INI_LIST(Connection,MonitorAddresses,CleanupList)
		READ_INI_LIST(Connection,DenyAddresses,CleanupList)
		READ_INI_RULE_CACHE(Connection,RequestsLimit)

		//Authentication
		READ_INI_BOOL(Authentication,ScanExcludedInstances)
		READ_INI_RULE(Authentication,BlankPasswords)
		READ_INI_RULE_LIST(Authentication,DenyDefaultPasswords,CleanupList)
		READ_INI_RULE(Authentication,SamePasswordAsUsername)
		READ_INI_RULE(Authentication,SystemAccounts)
		READ_INI_RULE_CACHE(Authentication,BruteForceAttack)
		READ_INI_BOOL(Authentication,ScanAccountAllEvents)
		READ_INI_RULE_LIST(Authentication,AllowAccounts,CleanupList)
		READ_INI_RULE_LIST(Authentication,DenyAccounts,CleanupList)
		
		//Request Limits
		READ_INI_RULE_STRING(Limits,MaxContentLength)
		READ_INI_RULE_INT(Limits,MaxHTTPVersion)
		READ_INI_RULE_LIST(Limits,AllowHTTPVersions,CleanupListUCase)

		//URL Scanning
		READ_INI_RULE(URL,RFCCompliantURL)
		READ_INI_RULE(URL,RFCCompliantHTTPURL)
		READ_INI_BOOL(URL,UseRawScan)
		READ_INI_RULE(URL,EncodingExploits)
		READ_INI_RULE(URL,ParentPath)
		READ_INI_RULE(URL,TrailingDotInDir)
		READ_INI_RULE(URL,Backslash)
		READ_INI_RULE(URL,AlternateStream)
		READ_INI_RULE(URL,Escaping)
		READ_INI_RULE(URL,MultipleCGI)
		READ_INI_RULE_INT(URL,MaxLength)
		READ_INI_RULE_CHARACTERS(URL,Characters)
		READ_INI_RULE(URL,HighBitShellcode)
		READ_INI_RULE(URL,SpecialWhitespace)
		READ_INI_RULE_LIST(URL,DenySequences,CleanupListLCase)
		READ_INI_RULE_MAP(URL,DenyRegex,CleanupList)
		READ_INI_RULE_LIST(URL,AllowURL,CleanupListLCase)
		READ_INI_RULE_CACHE(URL,RequestsLimit)
		READ_INI_LIST(URL,ExcludedURL,CleanupList)

		//Mapped Path
		READ_INI_RULE(Path,ParentPath)
		READ_INI_RULE(Path,SpecialWhitespace)
		READ_INI_RULE(Path,Escaping)
		READ_INI_RULE(Path,Dot)
		READ_INI_RULE(Path,MultipleColons)
		READ_INI_RULE_CHARACTERS(Path,Characters)
		READ_INI_RULE_LIST(Path,AllowPaths,CleanupListLCase)
		
		//Requested File
		READ_INI_BOOL(Filename,UseRawScan)
		READ_INI_RULE_CHARACTERS(Filename,Characters)
		READ_INI_RULE(Filename,DefaultDocument)
		READ_INI_RULE_LIST(Filename,DenyFiles,CleanupListLCase)
		READ_INI_LIST(Filename,MonitorFiles,CleanupListLCase)
		
		//Requested Extension
		READ_INI_RULE_LIST(Extensions,AllowExtensions,CleanupListLCase)
		READ_INI_RULE_LIST(Extensions,DenyExtensions,CleanupListLCase)
		if(!IsInList("",Extensions.AllowExtensions.List))
			Extensions.AllowExtensions.List.AddHead(""); //always allow default document
		READ_INI_RULE_CACHE(Extensions,RequestsLimit)
		READ_INI_CSTRINGLIST(Extensions,LimitExtensions,CleanupListLCase)
		
		//HTTP Headers
		READ_INI_RULE(Headers,RFCCompliantHostHeader)
		READ_INI_RULE(Headers,SQLInjection)
		READ_INI_RULE(Headers,EncodingExploits)
		READ_INI_RULE(Headers,DirectoryTraversal)
		READ_INI_RULE(Headers,HighBitShellcode)
		READ_INI_RULE_INT(Headers,MaxLength)
		READ_INI_RULE_MAP(Headers,MaxHeaders,CleanupList)		//TODO: SETTINGS - Add ':' to header name
		READ_INI_RULE_LIST(Headers,DenyHeaders,CleanupList)
		READ_INI_RULE_LIST(Headers,DenySequences,CleanupListLCase)
		READ_INI_RULE_MAP(Headers,DenyRegex,CleanupList)

		//CONTENTTYPE
		READ_INI_RULE_LIST(ContentType,AllowContentTypes,CleanupList)
		if(!IsInList("",ContentType.AllowContentTypes.List))
			ContentType.AllowContentTypes.List.AddHead(""); //always allow no content-type
		READ_INI_RULE_LIST(ContentType,DenyContentTypes,CleanupList)

		//COOKIE
		READ_INI_RULE(Cookie,SQLInjection)
		READ_INI_RULE(Cookie,EncodingExploits)
		READ_INI_RULE(Cookie,DirectoryTraversal)
		READ_INI_RULE(Cookie,HighBitShellcode)
		READ_INI_RULE(Cookie,SpecialWhitespace)
		READ_INI_RULE_INT(Cookie,MaxVariableLength)
		READ_INI_RULE_LIST(Cookie,DenySequences,CleanupListLCase)

		//USER-AGENT
		READ_INI_RULE(UserAgent,Empty)
		READ_INI_RULE(UserAgent,NonRFC)
		READ_INI_RULE(UserAgent,HighBitShellcode)
		READ_INI_RULE(UserAgent,SpecialWhitespace)
		READ_INI_RULE_CHARACTERS(UserAgent,RequireCharacter)
		READ_INI_RULE_CACHE(UserAgent,Switching)
		READ_INI_RULE_LIST(UserAgent,DenyUserAgents,CleanupList)
		READ_INI_RULE_LIST(UserAgent,DenySequences,CleanupListLCase)
		READ_INI_LIST(UserAgent,Excluded,CleanupList)

		//REFERRER
		READ_INI_BOOL(Referrer,UseScan)
		READ_INI_RULE(Referrer,RFCCompliantURL)
		READ_INI_RULE(Referrer,RFCCompliantHTTPURL)
		READ_INI_RULE(Referrer,EncodingExploits)
		READ_INI_RULE(Referrer,HotLinking)
		READ_INI_CSTRINGLIST(Referrer,HotLinkingUrls,CleanupListLCase)
		READ_INI_CSTRINGLIST(Referrer,HotLinkingFileExtensions,CleanupListLCase)
		READ_INI_RULE_LIST(Referrer,HotLinkingAllowDomains,CleanupListLCase)
		READ_INI_RULE_LIST(Referrer,HotLinkingDenyDomains,CleanupListLCase)
		READ_INI_BOOL(Referrer,HotLinkingUseHostHeader)
		READ_INI_RULE(Referrer,HotLinkingBlankReferrer)
		READ_INI_RULE_CHARACTERS(Referrer,Characters)
		READ_INI_RULE(Referrer,HighBitShellcode)
		READ_INI_RULE(Referrer,SpecialWhitespace)
		READ_INI_RULE(Referrer,SQLInjection)
		READ_INI_RULE_LIST(Referrer,DenySequences,CleanupListLCase)

		//HTTP Verbs (Request Methods)
		READ_INI_RULE_LIST(Verbs,AllowVerbs,CleanupList)
		READ_INI_RULE_LIST(Verbs,DenyVerbs,CleanupListUCase)

		//SQL Injection Keywords (TODO: ';' cannot be read again)
		READ_INI_INT(SQLi,AllowedCount)
		//l = GetPrivateProfileSection("SQLInjection.Keywords",buf,sz,fn);
		//Split(buf,l,SQLi.Keywords);
		//CleanupListLCase(SQLi.Keywords);

		//Encoding Exploits
		READ_INI_BOOL(EncodingExploits,ScanDoubleEncoding)
		l = GetPrivateProfileSection("EncodingExploits.Keywords",buf,sz,fn);
		Split(buf,l,EncodingExploits.Keywords);

		//Querystring Scanning
		READ_INI_BOOL(QueryString,UseRawScan)
		READ_INI_RULE(QueryString,SQLInjection)
		READ_INI_RULE(QueryString,EncodingExploits)
		READ_INI_RULE(QueryString,DirectoryTraversal)
		READ_INI_RULE(QueryString,HighBitShellcode)
		READ_INI_RULE(QueryString,SpecialWhitespace)
		READ_INI_RULE_INT(QueryString,MaxLength)
		READ_INI_RULE_INT(QueryString,MaxVariableLength)
		READ_INI_RULE_LIST(QueryString,DenySequences,CleanupListLCase)
		READ_INI_RULE_MAP(QueryString,DenyRegex,CleanupList)
		READ_INI_LIST(QueryString,ExcludedQueryString,CleanupList)

		//Global Filter Capabilities
		READ_INI_BOOL(Global,IsInstalledAsGlobalFilter)
		READ_INI_BOOL(Global,IsInstalledInWebProxy)
		READ_INI_RULE(Global,SlowHeaderAttack)
		READ_INI_RULE(Global,SlowPostAttack)
		READ_INI_RULE(Global,PostSQLInjection)
		READ_INI_RULE(Global,PostEncodingExploits)
		READ_INI_RULE(Global,PostDirectoryTraversal)
		READ_INI_RULE(Global,PostHighBitShellcode)
		READ_INI_RULE_INT(Global,MaxPostVariableLength)
		READ_INI_RULE_LIST(Global,DenyPostSequences,CleanupListLCase)
		READ_INI_RULE_MAP(Global,DenyPostRegex,CleanupList)

		//RESPONSE MONITOR
		READ_INI_BOOL(ResponseMonitor,RemoveServerHeader)
		READ_INI_BOOL(ResponseMonitor,ChangeServerHeader)
		READ_INI_STRING(ResponseMonitor,ServerHeader)
		READ_INI_MAP(ResponseMonitor,Headers,);
		READ_INI_BOOL(ResponseMonitor,LogClientErrors)
		READ_INI_RULE_CACHE(ResponseMonitor,ClientErrors)
		READ_INI_BOOL(ResponseMonitor,LogServerErrors)
		READ_INI_RULE_CACHE(ResponseMonitor,ServerErrors)
		READ_INI_RULE_MAP(ResponseMonitor,InformationDisclosure,CleanupList)

		//ADMIN
		READ_INI_BOOL(Admin,Enabled)
		READ_INI_CSTRINGLIST(Admin,FromIP,CleanupList) 
		READ_INI_BOOL(Admin,AllowConfigChanges)

		return true;
	}catch(...){
		LoadDefaults();
		return false;
	}
}

bool CHTTPFirewallSettings::ParseXML(CString &key, CString &val)
{
	//if not a valid key
	if (key=="")
		return false;

	if(!CFirewallSettings::ParseXML(key,val)){

		CString name = XMLDecode(XMLAttributeValue(key,"name"));

		bool done(true);

		//Scanning Engine
		PARSE_XML_BOOL(Engine,AllowLateScanning,"Allow_Late_Scanning")
		else PARSE_XML_BOOL(Engine,ScanNonSecurePort,"Scan_Non_Secure_Port")
		else PARSE_XML_BOOL(Engine,ScanSecurePort,"Scan_Secure_Port")
		else PARSE_XML_LIST(Engine,ExcludedInstances,"Excluded_Web_Instances")
		else PARSE_XML_LIST(Engine,ExcludedHosts,"Excluded_Host_Headers")
		else PARSE_XML_LIST(Engine,ExcludedIPAddresses,"Excluded_IP_Addresses")

		//Response Handling
		else PARSE_XML_BOOL(HTTPResponse,Directly,"Response_Directly")
		else PARSE_XML_BOOL(HTTPResponse,Redirect,"Response_Redirect")
		else PARSE_XML_STRING(HTTPResponse,RedirectURL,"Response_Redirect_URL")
		else PARSE_XML_BOOL(HTTPResponse,UseStatus,"Use_Response_Status")
		else PARSE_XML_STRING(HTTPResponse,Status,"Response_Status")
		else PARSE_XML_BOOL(HTTPResponse,DropConnection,"Response_Drop_Connection")
		else PARSE_XML_BOOL(HTTPResponse,MonitorIP,"Response_Monitor_IP")
		else PARSE_XML_INT(HTTPResponse,MonitorIPTimeout,"Response_Monitor_IP_Timeout",)
		else PARSE_XML_CACHE(HTTPResponse,BlockIP,"Response_Block_IP")

		//Logging
		else PARSE_XML_BOOL(HTTPLogging,Log_HTTP_VIA,"Log_HTTP_VIA")
		else PARSE_XML_BOOL(HTTPLogging,Log_HTTP_X_FORWARDED_FOR,"Log_HTTP_X_FORWARDED_FOR")
		else PARSE_XML_BOOL(HTTPLogging,LogHostHeader,"Log_Host_Header")
		else PARSE_XML_BOOL(HTTPLogging,LogUserAgent,"Log_User_Agent")

		//Connection
		else PARSE_XML_STRING(Connection,ClientIPVariable,"Connection_Client_IP_Variable")
		else PARSE_XML_BOOL(Connection,ChangeLogIPVariable,"Change_Log_IP_Variable")
		else PARSE_XML_LIST(Connection,MonitorAddresses,"Monitored_IP_Addresses")
		else PARSE_XML_LIST(Connection,DenyAddresses,"Denied_IP_Addresses")
		else PARSE_XML_RULE_CACHE(Connection,RequestsLimit,"Connection_Requests_Limit")

		//Authentication
		else PARSE_XML_BOOL(Authentication,ScanExcludedInstances,"Scan_Authentication_Excluded_Web_Instances")
		else PARSE_XML_RULE_DENY(Authentication,BlankPasswords,"Blank_Passwords")
		else PARSE_XML_RULE_DENY(Authentication,SamePasswordAsUsername,"Same_Password_As_Username")
		else PARSE_XML_RULE_LIST_USE(Authentication,DenyDefaultPasswords,"Denied_Default_Passwords")
		else PARSE_XML_RULE_LIST_USE(Authentication,DenyDefaultPasswords,"Default_Passwords")
		else PARSE_XML_RULE_DENY(Authentication,SystemAccounts,"System_Accounts")
		else PARSE_XML_RULE_CACHE(Authentication,BruteForceAttack,"Deny_Account_Brute_Force_Attack")//for backwards compatibility with WebKnight 3.2 and previous
		else PARSE_XML_RULE_CACHE(Authentication,BruteForceAttack,"Account_Brute_Force_Attack")
		else PARSE_XML_RULE_LIST_USE(Authentication,AllowAccounts,"Allowed_Accounts")
		else PARSE_XML_RULE_LIST_USE(Authentication,DenyAccounts,"Denied_Accounts")
		else PARSE_XML_BOOL(Authentication,ScanAccountAllEvents,"Scan_Account_All_Events")
			
		//Request Limits
		else PARSE_XML_RULE_STRING_LIMIT(Limits,MaxContentLength,"Content_Length")
		else PARSE_XML_RULE_INT_LIMIT(Limits,MaxHTTPVersion,"HTTP_Version")
		else PARSE_XML_RULE_LIST_USE(Limits,AllowHTTPVersions,"Allowed_HTTP_Versions")

		//performance tuning
		else{	done = false;	}
		if(!done){	
		done = true;

		//URL Scanning
		PARSE_XML_RULE(URL,RFCCompliantURL,"RFC_Compliant_Url")
		else PARSE_XML_RULE(URL,RFCCompliantHTTPURL,"RFC_Compliant_HTTP_Url")
		else PARSE_XML_BOOL(URL,UseRawScan,"Use_Url_Raw_Scan")
		else PARSE_XML_RULE_DENY(URL,EncodingExploits,"Url_Encoding_Exploits")
		else PARSE_XML_RULE_DENY(URL,ParentPath,"Url_Parent_Path")
		else PARSE_XML_RULE_DENY(URL,TrailingDotInDir,"Url_Trailing_Dot_In_Dir")
		else PARSE_XML_RULE_DENY(URL,Backslash,"Url_Backslash")
		else PARSE_XML_RULE_DENY(URL,AlternateStream,"Url_Alternate_Stream")
		else PARSE_XML_RULE_DENY(URL,Escaping,"Url_Escaping")
		else PARSE_XML_RULE_DENY(URL,MultipleCGI,"Url_Running_Multiple_CGI")
		else PARSE_XML_RULE_CHARACTERS_DENY(URL,Characters,"Url_Characters")
		else PARSE_XML_RULE(URL,HighBitShellcode,"Deny_Url_HighBitShellcode") //for backward compatibility with WebKnight 2.5 and previous
		else PARSE_XML_RULE_DENY(URL,HighBitShellcode,"Url_High_Bit_Shellcode")
		else PARSE_XML_RULE_DENY(URL,SpecialWhitespace,"Url_Special_Whitespace")
		else PARSE_XML_RULE_INT_LIMIT(URL,MaxLength,"Url")

		else PARSE_XML_RULE_LIST_USE(URL,DenySequences,"Denied_Url_Sequences")
		else PARSE_XML_CSTRINGLIST(URL,DenySequences.List,"URL_Denied_Sequences") //for backward compatibility with WebKnight 3.2 and previous

		else PARSE_XML_RULE_MAP(URL,DenyRegex,"Denied_Url_Regular_Expressions")

		else PARSE_XML_BOOL(URL,AllowURL.Action,"UseAllowedUrlStarts")		//for backward compatibility with WebKnight 1.0 & 1.1
		else PARSE_XML_RULE_LIST_USE(URL,AllowURL,"Allowed_Url_Starts")
		else PARSE_XML_CSTRINGLIST(URL,AllowURL.List,"Url_Allowed_starts")	//for backward compatibility with WebKnight 3.2 and previous
		else PARSE_XML_RULE_CACHE(URL,RequestsLimit,"Url_Requests_Limit")
		else PARSE_XML_LIST(URL,ExcludedURL,"Excluded_Urls")
				
		//Mapped Path
		else PARSE_XML_RULE_DENY(Path,ParentPath,"Parent_Path")
		else PARSE_XML_RULE_DENY(Path,SpecialWhitespace,"Special_Whitespace")
		else PARSE_XML_RULE(Path,SpecialWhitespace,"Deny_Backspace")		//for backward compatibility with WebKnight 2.5 and previous
		else PARSE_XML_RULE(Path,SpecialWhitespace,"Deny_Carriage_Return")	//for backward compatibility with WebKnight 2.5 and previous
		else PARSE_XML_RULE(Path,SpecialWhitespace,"Deny_New_Line")			//for backward compatibility with WebKnight 2.5 and previous
		else PARSE_XML_RULE_DENY(Path,Escaping,"Escaping")
		else PARSE_XML_RULE_DENY(Path,Dot,"Dot_In_Path")
		else PARSE_XML_RULE_DENY(Path,MultipleColons,"Multiple_Colons")
		else PARSE_XML_RULE_CHARACTERS_DENY(Path,Characters,"Characters")
		else PARSE_XML_RULE_LIST_USE(Path,AllowPaths,"Allowed_Paths")

		//performance tuning
		else{	done = false;	}
		if(!done){
		done = true;

		//Requested File
		PARSE_XML_BOOL(Filename,UseRawScan,"Use_Filename_Raw_Scan")
		else PARSE_XML_RULE_CHARACTERS_DENY(Filename,Characters,"Filename_Characters")
		else PARSE_XML_RULE_DENY(Filename,DefaultDocument,"Default_Document")
		else PARSE_XML_RULE_LIST_USE(Filename,DenyFiles,"Denied_Files")
		else PARSE_XML_LIST(Filename,MonitorFiles,"Monitored_Files")

		//Requested Extension
		else PARSE_XML_RULE_LIST_USE(Extensions,AllowExtensions,"Allowed_Extensions")
		else PARSE_XML_RULE_LIST_USE(Extensions,DenyExtensions,"Denied_Extensions")
		else PARSE_XML_RULE_CACHE(Extensions,RequestsLimit,"Extension_Requests_Limit")
		else PARSE_XML_CSTRINGLIST(Extensions,LimitExtensions,"Limit_Extensions")

		//HTTP Headers
		else PARSE_XML_RULE(Headers,RFCCompliantHostHeader,"RFC_Compliant_Host_Header")
		else PARSE_XML_RULE_DENY(Headers,SQLInjection,"Header_SQL_Injection")
		else PARSE_XML_RULE_DENY(Headers,EncodingExploits,"Header_Encoding_Exploits")
		else PARSE_XML_RULE_DENY(Headers,DirectoryTraversal,"Header_Directory_Traversal")
		else PARSE_XML_RULE_DENY(Headers,HighBitShellcode,"Header_High_Bit_Shellcode")
		else PARSE_XML_RULE_INT(Headers,MaxLength,"Maximum_Header_Length")
		else PARSE_XML_RULE_MAP(Headers,MaxHeaders,"Max_Headers")
		else PARSE_XML_RULE_LIST_USE(Headers,DenyHeaders,"Denied_Headers")
		else PARSE_XML_RULE_LIST_USE(Headers,DenySequences,"Denied_Header_Sequences")
		else PARSE_XML_RULE_MAP(Headers,DenyRegex,"Denied_Header_Regular_Expressions")

		//CONTENTYPE
		else PARSE_XML_RULE_LIST_USE(ContentType,AllowContentTypes,"Allowed_Content_Types")
		else PARSE_XML_RULE_LIST_USE(ContentType,DenyContentTypes,"Deny_Content_Types") //for backwards compatibility with WebKnight 2.2
		else PARSE_XML_RULE_LIST_USE(ContentType,DenyContentTypes,"Denied_Content_Types")

		//COOKIE
		else PARSE_XML_RULE_DENY(Cookie,SQLInjection,"Cookie_SQL_Injection")
		else PARSE_XML_RULE_DENY(Cookie,EncodingExploits,"Cookie_Encoding_Exploits")
		else PARSE_XML_RULE_DENY(Cookie,DirectoryTraversal,"Cookie_Directory_Traversal")
		else PARSE_XML_RULE_DENY(Cookie,HighBitShellcode,"Cookie_High_Bit_Shellcode")
		else PARSE_XML_RULE_DENY(Cookie,SpecialWhitespace,"Cookie_Special_Whitespace")
		else PARSE_XML_RULE_INT_BACKWARDS(Cookie,MaxVariableLength,"Maximum_Cookie_Variable_Length")
		else PARSE_XML_RULE_LIST_USE(Cookie,DenySequences,"Denied_Cookie_Sequences")
		
		//performance tuning
		else{	done = false;	}
		if(!done){
		done = true;

		//USER-AGENT
		PARSE_XML_RULE_DENY(UserAgent,Empty,"User_Agent_Empty")
		else PARSE_XML_RULE_DENY(UserAgent,NonRFC,"User_Agent_Non_RFC")
		else PARSE_XML_RULE_DENY(UserAgent,HighBitShellcode,"User_Agent_High_Bit_Shellcode")
		else PARSE_XML_RULE_DENY(UserAgent,SpecialWhitespace,"User_Agent_Special_Whitespace")
		else PARSE_XML_RULE_CHARACTERS_BACKWARDS(UserAgent,RequireCharacter,"Require_User_Agent_Character")
		else PARSE_XML_RULE_CACHE(UserAgent,Switching,"Block_User_Agent_Switching")//for backwards compatibility with WebKnight 3.2 and previous
		else PARSE_XML_RULE_CACHE(UserAgent,Switching,"User_Agent_Switching")
		else PARSE_XML_RULE_LIST_USE(UserAgent,DenyUserAgents,"Denied_User_Agents")
		else PARSE_XML_RULE_LIST_USE(UserAgent,DenySequences,"Denied_User_Agent_Sequences")
		else PARSE_XML_LIST(UserAgent,Excluded,"Excluded_User_Agents")

		//REFERRER
		else PARSE_XML_BOOL(Referrer,UseScan,"Use_Referrer_Scanning")
		else PARSE_XML_RULE(Referrer,RFCCompliantURL,"Referrer_URL_RFC_Compliant")
		else PARSE_XML_RULE(Referrer,RFCCompliantHTTPURL,"Referrer_URL_RFC_HTTPCompliant")
		else PARSE_XML_RULE_DENY(Referrer,EncodingExploits,"Referrer_Encoding_Exploits")
		else PARSE_XML_RULE_DENY(Referrer,HotLinking,"Referrer_Hot_Linking")
		else PARSE_XML_CSTRINGLIST(Referrer,HotLinkingUrls,"Referrer_Hot_Linking_Urls")
		else PARSE_XML_CSTRINGLIST(Referrer,HotLinkingFileExtensions,"Referrer_Hot_Linking_File_Extensions")
		else PARSE_XML_RULE_LIST_USE(Referrer,HotLinkingAllowDomains,"Referrer_Hot_Linking_Allow_Domains")
		else PARSE_XML_RULE_LIST_USE(Referrer,HotLinkingDenyDomains,"Referrer_Hot_Linking_Deny_Domains")
		else PARSE_XML_BOOL(Referrer,HotLinkingUseHostHeader,"Referrer_Hot_Linking_Use_Host_Header")
		else PARSE_XML_RULE(Referrer,HotLinkingBlankReferrer,"Referrer_Hot_Linking_Deny_Blank_Referrer") //for backwards compatibility with WebKnight 4.0 and earlier
		else PARSE_XML_RULE(Referrer,HotLinkingBlankReferrer,"Referrer_Hot_Linking_Blank_Referrer")
		else PARSE_XML_RULE_CHARACTERS_DENY(Referrer,Characters,"Referrer_Characters")
		else PARSE_XML_RULE_DENY(Referrer,HighBitShellcode,"Referrer_High_Bit_Shellcode")
		else PARSE_XML_RULE_DENY(Referrer,SpecialWhitespace,"Referrer_Special_Whitespace")
		else PARSE_XML_RULE_DENY(Referrer,SQLInjection,"Referrer_SQL_Injection")
		else PARSE_XML_RULE_LIST_USE(Referrer,DenySequences,"Deny_Referrer_Sequences")

		else { done = false; }
		if(!done){
			
			//HTTP Verbs (Request Methods)
			PARSE_XML_RULE_LIST_USE(Verbs,AllowVerbs,"Allowed_Verbs")
			else PARSE_XML_RULE_LIST_USE(Verbs,DenyVerbs,"Denied_Verbs")

			//Querystring Scanning
			else PARSE_XML_BOOL(QueryString,UseRawScan,"Use_Querystring_Raw_Scan")
			else PARSE_XML_RULE_DENY(QueryString,SQLInjection,"Querystring_SQL_Injection")
			else PARSE_XML_RULE_DENY(QueryString,EncodingExploits,"Querystring_Encoding_Exploits")
			else PARSE_XML_RULE_DENY(QueryString,DirectoryTraversal,"Querystring_Directory_Traversal")
			else PARSE_XML_RULE_DENY(QueryString,HighBitShellcode,"Querystring_High_Bit_Shellcode")
			else PARSE_XML_RULE_DENY(QueryString,SpecialWhitespace,"Querystring_Special_Whitespace")
			else PARSE_XML_RULE_INT_LIMIT(QueryString,MaxLength,"Querystring")
			else PARSE_XML_RULE_INT_BACKWARDS(QueryString,MaxVariableLength,"Maximum_Querystring_Variable_Length")
			else PARSE_XML_RULE_LIST_USE(QueryString,DenySequences,"Denied_Querystring_Sequences")
			else PARSE_XML_RULE_MAP(QueryString,DenyRegex,"Denied_Querystring_Regular_Expressions")
			else PARSE_XML_LIST(QueryString,ExcludedQueryString,"Excluded_Querystrings")
					
			//Global Filter Capabilities
			else PARSE_XML_BOOL(Global,IsInstalledAsGlobalFilter,"Is_Installed_As_Global_Filter")
			else PARSE_XML_BOOL(Global,IsInstalledInWebProxy,"Is_Installed_In_Web_Proxy")
			else PARSE_XML_RULE_DENY(Global,SlowHeaderAttack,"Slow_Header_Attack")
			else PARSE_XML_RULE_DENY(Global,SlowPostAttack,"Slow_Post_Attack")
			else PARSE_XML_RULE_DENY(Global,PostSQLInjection,"Postdata_SQL_Injection")
			else PARSE_XML_RULE_DENY(Global,PostEncodingExploits,"Postdata_Encoding_Exploits")
			else PARSE_XML_RULE_DENY(Global,PostDirectoryTraversal,"Postdata_Directory_Traversal")
			else PARSE_XML_RULE_DENY(Global,PostHighBitShellcode,"Postdata_High_Bit_Shellcode")
			else PARSE_XML_RULE_INT_BACKWARDS(Global,MaxPostVariableLength,"Maximum_Postdata_Variable_Length")
			else PARSE_XML_RULE_LIST_USE(Global,DenyPostSequences,"Denied_Post_Sequences")
			else PARSE_XML_RULE_MAP(Global,DenyPostRegex,"Denied_Post_Regular_Expressions")

			//Response Monitoring
			else PARSE_XML_BOOL(ResponseMonitor,RemoveServerHeader,"Remove_Server_Header")
			else PARSE_XML_BOOL(ResponseMonitor,ChangeServerHeader,"Change_Server_Header")
			else PARSE_XML_STRING(ResponseMonitor,ServerHeader,"Server_Header")
			else PARSE_XML_MAP(ResponseMonitor,Headers,"Response_Headers")
			else PARSE_XML_BOOL(ResponseMonitor,LogClientErrors,"Log_HTTP_Client_Errors")
			else PARSE_XML_RULE_CACHE(ResponseMonitor,ClientErrors,"Block_HTTP_Client_Errors")//for backwards compatibility with WebKnight 3.2 and previous
			else PARSE_XML_RULE_CACHE(ResponseMonitor,ClientErrors,"HTTP_Client_Errors")
			else PARSE_XML_BOOL(ResponseMonitor,LogServerErrors,"Log_HTTP_Server_Errors")
			else PARSE_XML_RULE_CACHE(ResponseMonitor,ServerErrors,"Block_HTTP_Server_Errors")//for backwards compatibility with WebKnight 3.2 and previous
			else PARSE_XML_RULE_CACHE(ResponseMonitor,ServerErrors,"HTTP_Server_Errors")
			else PARSE_XML_RULE_MAP_PREFIX(ResponseMonitor,InformationDisclosure,"Information_Disclosure","Deny_")

			//ADMIN
			else PARSE_XML_BOOL(Admin,Enabled,"Admin_Enabled")
			else PARSE_XML_CSTRINGLIST(Admin,FromIP,"Admin_From_IP")
			else PARSE_XML_BOOL(Admin,AllowConfigChanges,"Admin_Allow_Configuration_Changes")

			//Encoding Exploits
			else PARSE_XML_BOOL(EncodingExploits,ScanDoubleEncoding,"Detect_Double_Encoding")
			else if(name.CompareNoCase("Encoding_Keywords")==0){
				ConvertXMLToList(val,EncodingExploits.Keywords);
			}

			//SQL Injection Keywords
			else PARSE_XML_INT(SQLi,AllowedCount,"SQL_Injection_Allowed_Count",)
			else if(name.CompareNoCase("SQL_Injection_Keywords")==0){
				ConvertXMLToList(val,SQLi.Keywords);
			}else{
				return false; //not parsed
			}
		}
		}
		}
		}
	}
	return true;
}

bool CHTTPFirewallSettings::CheckConstraints()
{
	//base class check
	CFirewallSettings::CheckConstraints();

	//lowercase
	LCase(SQLi.Keywords);
	LCase(Headers.DenySequences.List);
	LCase(Global.DenyPostSequences.List);
	LCase(URL.AllowURL.List);
	LCase(URL.DenySequences.List);
	LCase(Path.AllowPaths.List);
	LCase(QueryString.DenySequences.List);
	LCase(Filename.DenyFiles.List);
	LCase(Extensions.AllowExtensions.List);
	LCase(Extensions.DenyExtensions.List);
	LCase(Extensions.LimitExtensions);
	LCase(UserAgent.DenySequences.List);
	LCase(Filename.MonitorFiles.List);

	LCase(Referrer.HotLinkingUrls);
	LCase(Referrer.HotLinkingFileExtensions);
	LCase(Referrer.HotLinkingAllowDomains.List);
	LCase(Referrer.HotLinkingDenyDomains.List);
	LCase(Referrer.DenySequences.List);

	//uppercase
	UCase(Verbs.DenyVerbs.List);

	return true;
}

bool CHTTPFirewallSettings::WriteToFileINI(LPCTSTR fn)
{
	/* 
	 * Write all settings to a simple INI file
	 * All checks on the file should have been
	 * done before this function is called
	 */
	try{

	//Scanning engine
	WRITE_INI_BOOL(Engine,AllowLateScanning)
	WRITE_INI_BOOL(Engine,ScanNonSecurePort)
	WRITE_INI_BOOL(Engine,ScanSecurePort)
	WRITE_INI_LIST(Engine,ExcludedInstances)
	WRITE_INI_LIST(Engine,ExcludedHosts)
	WRITE_INI_LIST(Engine,ExcludedIPAddresses)

	//Response Handling
	const _HTTPResponse& Response = HTTPResponse;
	WRITE_INI_BOOL(Response,Directly)
	WRITE_INI_BOOL(Response,Redirect)
	WRITE_INI_STRING(Response,RedirectURL)
	WRITE_INI_BOOL(Response,UseStatus)
	WRITE_INI_STRING(Response,Status)
	WRITE_INI_BOOL(Response,DropConnection)
	WRITE_INI_BOOL(Response,MonitorIP)
	WRITE_INI_INT(Response,MonitorIPTimeout)
	WRITE_INI_CACHE(Response,BlockIP)

	CFirewallSettings::WriteToFileINI(fn);

	//Logging
	const _HTTPLogging& Logging = HTTPLogging;
	WRITE_INI_BOOL(Logging,Log_HTTP_VIA)
	WRITE_INI_BOOL(Logging,Log_HTTP_X_FORWARDED_FOR)
	WRITE_INI_BOOL(Logging,LogHostHeader)
	WRITE_INI_BOOL(Logging,LogUserAgent)

	//Connection control
	WRITE_INI_STRING(Connection,ClientIPVariable)
	WRITE_INI_BOOL(Connection,ChangeLogIPVariable)
	WRITE_INI_LIST(Connection,MonitorAddresses)
	WRITE_INI_LIST(Connection,DenyAddresses)
	WRITE_INI_RULE_CACHE(Connection,RequestsLimit)
	
	//Authentication
	WRITE_INI_BOOL(Authentication,ScanExcludedInstances)
	WRITE_INI_RULE(Authentication,BlankPasswords)
	WRITE_INI_RULE(Authentication,SamePasswordAsUsername)
	WRITE_INI_RULE_LIST(Authentication,DenyDefaultPasswords)
	WRITE_INI_RULE(Authentication,SystemAccounts)
	WRITE_INI_RULE_CACHE(Authentication,BruteForceAttack)
	WRITE_INI_BOOL(Authentication,ScanAccountAllEvents)
	WRITE_INI_RULE_LIST(Authentication,AllowAccounts)
	WRITE_INI_RULE_LIST(Authentication,DenyAccounts)

	//Request Limits
	WRITE_INI_RULE_STRING(Limits,MaxContentLength)
	WRITE_INI_RULE_INT(Limits,MaxHTTPVersion)
	WRITE_INI_RULE_LIST(Limits,AllowHTTPVersions)

	//URL
	WRITE_INI_RULE(URL,RFCCompliantURL)
	WRITE_INI_RULE(URL,RFCCompliantHTTPURL)
	WRITE_INI_BOOL(URL,UseRawScan)
	WRITE_INI_RULE(URL,EncodingExploits)
	WRITE_INI_RULE(URL,ParentPath)
	WRITE_INI_RULE(URL,TrailingDotInDir)
	WRITE_INI_RULE(URL,Backslash)
	WRITE_INI_RULE(URL,AlternateStream)
	WRITE_INI_RULE(URL,Escaping)
	WRITE_INI_RULE(URL,MultipleCGI)
	WRITE_INI_RULE_INT(URL,MaxLength)
	WRITE_INI_RULE_CHARACTERS(URL,Characters)
	WRITE_INI_RULE(URL,HighBitShellcode)
	WRITE_INI_RULE(URL,SpecialWhitespace)
	WRITE_INI_RULE_LIST(URL,DenySequences)
	WRITE_INI_RULE_MAP(URL,DenyRegex)
	WRITE_INI_RULE_LIST(URL,AllowURL)
	WRITE_INI_RULE_CACHE(URL,RequestsLimit)
	WRITE_INI_LIST(URL,ExcludedURL)

	//PATH
	WRITE_INI_RULE(Path,ParentPath)
	WRITE_INI_RULE(Path,SpecialWhitespace)
	WRITE_INI_RULE(Path,Escaping)
	WRITE_INI_RULE(Path,Dot)
	WRITE_INI_RULE(Path,MultipleColons)
	WRITE_INI_RULE_CHARACTERS(Path,Characters)
	WRITE_INI_RULE_LIST(Path,AllowPaths)

	//FILENAME
	WRITE_INI_BOOL(Filename,UseRawScan)
	WRITE_INI_RULE_CHARACTERS(Filename,Characters)
	WRITE_INI_RULE(Filename,DefaultDocument)
	WRITE_INI_RULE_LIST(Filename,DenyFiles)
	WRITE_INI_LIST(Filename,MonitorFiles)
		
	//EXTENSION
	WRITE_INI_RULE_LIST(Extensions,AllowExtensions)
	WRITE_INI_RULE_LIST(Extensions,DenyExtensions)
	WRITE_INI_RULE_CACHE(Extensions,RequestsLimit)
	WRITE_INI_CSTRINGLIST(Extensions,LimitExtensions)

	//HEADERS
	WRITE_INI_RULE(Headers,RFCCompliantHostHeader)
	WRITE_INI_RULE(Headers,SQLInjection)
	WRITE_INI_RULE(Headers,EncodingExploits)
	WRITE_INI_RULE(Headers,DirectoryTraversal)
	WRITE_INI_RULE(Headers,HighBitShellcode)
	WRITE_INI_RULE_INT(Headers,MaxLength)
	WRITE_INI_RULE_MAP(Headers,MaxHeaders)
	WRITE_INI_RULE_LIST(Headers,DenyHeaders)
	WRITE_INI_RULE_LIST(Headers,DenySequences)
	WRITE_INI_RULE_MAP(Headers,DenyRegex)

	//CONTENTTYPE
	WRITE_INI_RULE_LIST(ContentType,AllowContentTypes)
	WRITE_INI_RULE_LIST(ContentType,DenyContentTypes)

	//COOKIE
	WRITE_INI_RULE(Cookie,SQLInjection)
	WRITE_INI_RULE(Cookie,EncodingExploits)
	WRITE_INI_RULE(Cookie,DirectoryTraversal)
	WRITE_INI_RULE(Cookie,HighBitShellcode)
	WRITE_INI_RULE(Cookie,SpecialWhitespace)
	WRITE_INI_RULE_INT(Cookie,MaxVariableLength)
	WRITE_INI_RULE_LIST(Cookie,DenySequences)

	//USER-AGENT
	WRITE_INI_RULE(UserAgent,Empty)
	WRITE_INI_RULE(UserAgent,NonRFC)
	WRITE_INI_RULE(UserAgent,HighBitShellcode)
	WRITE_INI_RULE(UserAgent,SpecialWhitespace)
	WRITE_INI_RULE_CHARACTERS(UserAgent,RequireCharacter)
	WRITE_INI_RULE_CACHE(UserAgent,Switching)
	WRITE_INI_RULE_LIST(UserAgent,DenyUserAgents)
	WRITE_INI_RULE_LIST(UserAgent,DenySequences)
	WRITE_INI_LIST(UserAgent,Excluded)

	//REFERRER
	WRITE_INI_BOOL(Referrer,UseScan)
	WRITE_INI_RULE(Referrer,RFCCompliantURL)
	WRITE_INI_RULE(Referrer,RFCCompliantHTTPURL)
	WRITE_INI_RULE(Referrer,EncodingExploits)
	WRITE_INI_RULE(Referrer,HotLinking)
	WRITE_INI_CSTRINGLIST(Referrer,HotLinkingUrls)
	WRITE_INI_CSTRINGLIST(Referrer,HotLinkingFileExtensions)
	WRITE_INI_RULE_LIST(Referrer,HotLinkingAllowDomains)
	WRITE_INI_RULE_LIST(Referrer,HotLinkingDenyDomains)
	WRITE_INI_BOOL(Referrer,HotLinkingUseHostHeader)
	WRITE_INI_RULE(Referrer,HotLinkingBlankReferrer)
	WRITE_INI_RULE_CHARACTERS(Referrer,Characters)
	WRITE_INI_RULE(Referrer,HighBitShellcode)
	WRITE_INI_RULE(Referrer,SpecialWhitespace)
	WRITE_INI_RULE(Referrer,SQLInjection)
	WRITE_INI_RULE_LIST(Referrer,DenySequences)

	//VERBS
	WRITE_INI_RULE_LIST(Verbs,AllowVerbs)
	WRITE_INI_RULE_LIST(Verbs,DenyVerbs)

	//SQL Injection
	WRITE_INI_INT(SQLi,AllowedCount)
	//WritePrivateProfileSection("SQLInjection.Keywords",ConvertListToString(SQLKeyWords),fn);
	//TODO: SETTINGS - writing ';' to ini file cannot be read again

	//ENCODING EXPLOITS
	WRITE_INI_BOOL(EncodingExploits,ScanDoubleEncoding)
	WritePrivateProfileSection("EncodingExploits.Keywords",ConvertListToString(EncodingExploits.Keywords),fn);
	
	//QUERYSTRING
	WRITE_INI_BOOL(QueryString,UseRawScan)
	WRITE_INI_RULE(QueryString,SQLInjection)
	WRITE_INI_RULE(QueryString,EncodingExploits)
	WRITE_INI_RULE(QueryString,DirectoryTraversal)
	WRITE_INI_RULE(QueryString,HighBitShellcode)
	WRITE_INI_RULE(QueryString,SpecialWhitespace)
	WRITE_INI_RULE_INT(QueryString,MaxLength)
	WRITE_INI_RULE_INT(QueryString,MaxVariableLength)
	WRITE_INI_RULE_LIST(QueryString,DenySequences)
	WRITE_INI_RULE_MAP(QueryString,DenyRegex)
	WRITE_INI_LIST(QueryString,ExcludedQueryString)
	
	//Raw data (headers & POST data)
	WRITE_INI_BOOL(Global,IsInstalledAsGlobalFilter)
	WRITE_INI_BOOL(Global,IsInstalledInWebProxy)
	WRITE_INI_RULE(Global,SlowHeaderAttack)
	WRITE_INI_RULE(Global,SlowPostAttack)
	WRITE_INI_RULE(Global,PostSQLInjection)
	WRITE_INI_RULE(Global,PostEncodingExploits)
	WRITE_INI_RULE(Global,PostDirectoryTraversal)
	WRITE_INI_RULE(Global,PostHighBitShellcode)
	WRITE_INI_RULE_INT(Global,MaxPostVariableLength)
	WRITE_INI_RULE_LIST(Global,DenyPostSequences)
	WRITE_INI_RULE_MAP(Global,DenyPostRegex)

	//RESPONSE MONITOR
	//WRITE_INI_BOOL(ResponseMonitor,RemoveServerHeader)
	//WRITE_INI_BOOL(ResponseMonitor,ChangeServerHeader)
	//WRITE_INI_STRING(ResponseMonitor,ServerHeader)
	WRITE_INI_MAP(ResponseMonitor,Headers)
	WRITE_INI_BOOL(ResponseMonitor,LogClientErrors)
	WRITE_INI_RULE_CACHE(ResponseMonitor,ClientErrors)
	WRITE_INI_BOOL(ResponseMonitor,LogServerErrors)
	WRITE_INI_RULE_CACHE(ResponseMonitor,ServerErrors)
	WRITE_INI_RULE_MAP(ResponseMonitor,InformationDisclosure)

	//ADMIN
	WRITE_INI_BOOL(Admin,Enabled)
	WRITE_INI_CSTRINGLIST(Admin,FromIP) 
	WRITE_INI_BOOL(Admin,AllowConfigChanges)

		return true;
	}catch(...){
		return false;
	}
}

bool CHTTPFirewallSettings::WriteToFileXML(LPCTSTR fn)
{
	CString xml("");

	//xml += "<!DOCTYPE CISAPIFilter[\r\n";			//start dtd
	//xml += "<!ELEMENT CHTTPFirewallSettings ANY >\r\n";
	//xml += " ]>\r\n";							//end dtd

	xml += "<CHTTPFirewallSettings>\r\n";	//start entity
	xml += "<AQTRONIX_CHTTPFirewallSettings App='Title'/>\r\n";
	xml += ToXML();
	xml += "</CHTTPFirewallSettings>\r\n";		//end entity

	return WriteXMLEntityToFile(fn,xml);
}

CString CHTTPFirewallSettings::GetXMLAdmin(CHTTPFirewallSettings& d)
{
	CString xml("");
	//ADMIN
	xml += "<Admin App='Separator'/>\r\n";
	WRITE_XML_BOOL(Admin,Enabled,"Admin_Enabled","Enable the built-in admin web interface: " + Admin.Url)
	WRITE_XML_CSTRINGLIST(Admin,FromIP,"Admin_From_IP","The IP addresses or ranges allowed access to the built-in admin interface")
	WRITE_XML_BOOL(Admin,AllowConfigChanges,"Admin_Allow_Configuration_Changes","Allow the built-in admin web interface to make changes to the configuration (IUSR needs to have change permission on the configuration files). If disabled the admin will be a reporting utility only, no changes to the configuration files will be possible (you can also remove IUSR access). This feature requires ASP classic to be installed.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLScanningEngine(CHTTPFirewallSettings& d)
{
	CString xml("");
	//Scanning engine
	xml += "<Scanning_Engine App='Separator'/>\r\n";
	WRITE_XML_BOOL(Engine,AllowLateScanning,"Allow_Late_Scanning","This will run the filter as a low priority filter instead of high priority. Recommended is high priority (is more secure as it precedes other ISAPI filters which might have potential buffer overflows, etc.).\r\nRequires restart of IIS!")
	WRITE_XML_BOOL(Engine,ScanNonSecurePort,"Scan_Non_Secure_Port","Scan unencrypted (HTTP) web traffic (default port 80, but can be anything else).\r\nRequires restart of IIS!")
	WRITE_XML_BOOL(Engine,ScanSecurePort,"Scan_Secure_Port","Scan encrypted (HTTPS) web traffic (default port 443, but can be anything else).\r\nRequires restart of IIS!")
	WRITE_XML_LIST(Engine,ExcludedInstances,"Excluded_Web_Instances","Exclude certain web instances. Recommended for web instances which have their own ISAPI filter like Outlook Web Access.","These are the web instances excluded from scanning. Examples are Outlook Web Access web sites (starting at site instance 100).")
	WRITE_XML_LIST(Engine,ExcludedHosts,"Excluded_Host_Headers","Exclude certain host headers, recommended if you want to exclude scanning requests for certain web sites","These are the 'Host:' header values excluded from scanning. Requests with these Host headers will not be scanned. Example: 'localhost'.")
	WRITE_XML_LIST(Engine,ExcludedIPAddresses,"Excluded_IP_Addresses","Exclude certain IP addresses or ranges, this allows certain hosts to have unfiltered access to your web services.","These are the IP addresses or ranges excluded from scanning.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLResponseHandling(CHTTPFirewallSettings& d)
{
	CString xml("");
	//Response Handling
	xml += "<Incident_Response_Handling App='Separator'/>\r\n";
	WRITE_XML_BOOL(HTTPResponse,Directly,"Response_Directly","If an attack is detected, send an immediate response to the client with a standard message. The message sent back is the contents of the denied.htm file (in the directory of the firewall)")
	WRITE_XML_BOOL(HTTPResponse,Redirect,"Response_Redirect","If an attack is detected, redirect the client to a custom url (the url specified in 'Response Redirect URL').")
	WRITE_XML_STRING(HTTPResponse,RedirectURL,"Response_Redirect_URL","This is the URL the client is redirected to if 'Response Redirect' is chosen and 'Response Directly' is disabled. This can be an absolute URL (like \"http://www.aqtronix.com\") or a relative one (like \"/denied.htm\").")
	WRITE_XML_BOOL(HTTPResponse,UseStatus,"Use_Response_Status","Whenever an attack is detected, use the value in 'Response Status' as the HTTP response status that will be sent back to the client. This only works if you don't redirect the client to a custom URL.")
	WRITE_XML_STRING(HTTPResponse,Status,"Response_Status","This is the HTTP response status like '31337 No Hacking' or '404 Object Not Found' that is sent back to the client when an attack is detected.")
	WRITE_XML_BOOL(HTTPResponse,DropConnection,"Response_Drop_Connection","Whenever an attack is detected, drop the existing connection (even if keep-alive was requested).")
	WRITE_XML_BOOL(HTTPResponse,MonitorIP,"Response_Monitor_IP","Monitor traffic coming from that IP address for a certain time-out period.")
	WRITE_XML_INT(HTTPResponse,MonitorIPTimeout,"Response_Monitor_IP_Timeout","The time-out (in hours) to monitor traffic coming from that IP address.")
	WRITE_XML_CACHE(HTTPResponse,BlockIP,"Response_Block_IP","Block the IP address if it generates too many alerts.","The maximum number of alerts before being blocked in a certain amount of time.","The time frame in which the alerts are counted (in hours).")
	xml += CFirewallSettings::GetXMLResponseHandling(d);
	return xml;
}

CString CHTTPFirewallSettings::GetXMLConnectionControl(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Connection control
	xml += "<Connection App='Separator'/>\r\n";
	WRITE_XML_STRING(Connection,ClientIPVariable,"Connection_Client_IP_Variable","The server variable to get the client IP address. Use this if the incoming requests are coming from a CDN/reverse proxy. Examples are: HTTP_X_FORWARDED_FOR or HTTP_TRUE_CLIENT_IP. If empty, REMOTE_ADDR is used.")
	WRITE_XML_BOOL(Connection,ChangeLogIPVariable,"Change_Log_IP_Variable","Adjust the web server log entries to reflect the IP address acquired from the custom IP variable.")
	WRITE_XML_LIST(Connection,MonitorAddresses,"Monitored_IP_Addresses","Monitor the traffic of certain IP addresses by logging their requests.","These are the monitored IP addresses and IP ranges. For ranges you can use wildcards ('10.*.*.*') and CIDR notation ('10.0.0.0/8') or hyphen ('10.0.0.1-10.0.0.5').")
	WRITE_XML_LIST(Connection,DenyAddresses,"Denied_IP_Addresses","Deny access from certain IP addresses and log their requests.","These are the denied IP addresses and IP ranges. For ranges you can use wildcards ('10.*.*.*') and CIDR notation ('10.0.0.0/8') or hyphen ('10.0.0.1-10.0.0.5').")
	WRITE_XML_RULE_CACHE(Connection,RequestsLimit,"Connection_Requests_Limit","Limit the number of requests an IP address can make.","The number of requests that can be made in a certain amount of time.","The time frame in which the requests are counted (in minutes).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLAuthentication(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Authentication
	xml += "<Authentication App='Separator'/>\r\n";
	WRITE_XML_BOOL(Authentication,ScanExcludedInstances,"Scan_Authentication_Excluded_Web_Instances","Also scan the excluded web instances in this event. Excluded web instances are not scanned by the firewall except for authentication attempts.")
	WRITE_XML_RULE(Authentication,BlankPasswords,"Blank_Passwords","This will block authentication attempts with blank passwords.")
	WRITE_XML_RULE(Authentication,SamePasswordAsUsername,"Same_Password_As_Username","This will block authentication attempts with passwords equal to the username.")
	WRITE_XML_RULE_LIST(Authentication,DenyDefaultPasswords,"Default_Passwords","This will block authentication attempts with default and most used passwords.","These are the default passwords that cannot be used to authenticate.")
	WRITE_XML_RULE(Authentication,SystemAccounts,"System_Accounts","This will block authentication attempts with a system critical account (like IUSR_SERVERNAME, IWAM_SERVERNAME, SYSTEM, NETWORK SERVICE, TsInternetUser...).")
	WRITE_XML_RULE_CACHE(Authentication,BruteForceAttack,"Account_Brute_Force_Attack","This will block brute force attacks and possible account lockout Denial-of-Service. Detecting this is done by counting the authentication attempts within a certain period.","The maximum number an IP address is allowed to authenticate within a certain time frame.","The time frame (in minutes) within the number of authentication attempts are counted.")
	WRITE_XML_RULE_LIST(Authentication,AllowAccounts,"Allowed_Accounts","Only allow authentication attempts for certain accounts.","These are the accounts allowed to authenticate.")
	WRITE_XML_RULE_LIST(Authentication,DenyAccounts,"Denied_Accounts","Block authentication attempts for certain accounts.","These are the accounts explicitly denied to authenticate.")
	WRITE_XML_BOOL(Authentication,ScanAccountAllEvents,"Scan_Account_All_Events","This will scan the used account (logged on user) in all other ISAPI events and possibly block the request if the account is not allowed to authenticate.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLRequestLimits(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Request Limits
	xml += "<Request_Limits App='Separator'/>\r\n";
	WRITE_XML_RULE_STRING(Limits,MaxContentLength,"Maximum_Content_Length","Limit the value of the Content-Length header in a request. This allows you to limit the number of bytes sent to the server in requests.")
	WRITE_XML_RULE_INT(Limits,MaxHTTPVersion,"Maximum_HTTP_Version","Limit the length of the HTTP version string. Every request to the web server involves specifying the HTTP version (like 'HTTP/1.1').")
	WRITE_XML_RULE_LIST(Limits,AllowHTTPVersions,"Allowed_HTTP_Versions","Only allow certain HTTP versions.","These are the allowed HTTP versions. An empty line means you allow the HTTP version 0.9 (no http version)")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLURLScanning(CHTTPFirewallSettings &d)
{
	CString xml("");
	//URL Scanning
	xml += "<URL_Scanning App='Separator'/>\r\n";
	WRITE_XML_RULE(URL,RFCCompliantURL,"RFC_Compliant_Url","Check if the URL is RFC compliant. If it is not the request will be blocked.")
	WRITE_XML_RULE(URL,RFCCompliantHTTPURL,"RFC_Compliant_HTTP_Url","Check if the HTTP URL is RFC compliant. This will block authentication in the HTTP url (absolute URLs only).")
	WRITE_XML_BOOL(URL,UseRawScan,"Use_Url_Raw_Scan","Besides using the default scanning, also use the raw scanning capability to scan the URL before the web server decodes the URL (with built-in decoding engine).")
	WRITE_XML_RULE(URL,EncodingExploits,"Url_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the URL.")
	WRITE_XML_RULE(URL,ParentPath,"Url_Parent_Path","Deny parent path ('..') attempt in the requested url.")
	WRITE_XML_RULE(URL,TrailingDotInDir,"Url_Trailing_Dot_In_Dir","Deny a trailing dot in a directory name. This will block all requests with './'.")
	WRITE_XML_RULE(URL,Backslash,"Url_Backslash","Deny backward slashes ('\\') in the url.")
	WRITE_XML_RULE(URL,AlternateStream,"Url_Alternate_Stream","This will block all requests with a ':' in the url.")
	WRITE_XML_RULE(URL,Escaping,"Url_Escaping","Do not allow '%' in the url after decoding. This will block encoding exploits (embedded encoding) in the url.")
	WRITE_XML_RULE(URL,MultipleCGI,"Url_Running_Multiple_CGI","Do not allow using the ampersand ('&') in a url. This can be used to run multiple CGI applications.")
	WRITE_XML_RULE_INT(URL,MaxLength,"Maximum_Url","Limit the length of the url (more precisely everything in the url before the '?'). Certain attacks involve long urls. You should not allow urls longer than the longest path your operating system allows.")
	WRITE_XML_RULE_CHARACTERS(URL,Characters,"Url_Characters","Additional characters to block. If a requested url contains one of these, the request will be blocked.")
	WRITE_XML_RULE(URL,HighBitShellcode,"Url_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended on non-US-English web sites. This will also block Unicode/UTF-8 and MBCS in the URL.")
	WRITE_XML_RULE(URL,SpecialWhitespace,"Url_Special_Whitespace","Do not allow carriage return, line feed, backspace and tabulator characters.")
	WRITE_XML_RULE_LIST(URL,DenySequences,"Denied_Url_Sequences","Block the request if the url contains one or more sequences.","These are the sequences not allowed in a url.")
	WRITE_XML_RULE_MAP(URL,DenyRegex,"Denied_Url_Regular_Expressions","Block the request if the url matches a certain regular expression.","The regular expressions to block. 'Key' is the name of the rule (will be logged). 'Value' is the regex pattern used to find matches (CAtlRegExp syntax).")
	WRITE_XML_RULE_LIST(URL,AllowURL,"Allowed_Url_Starts","Only allow certain characters a url may start with.","These are the allowed character sequences a url may start with.")
	WRITE_XML_RULE_CACHE(URL,RequestsLimit,"Url_Requests_Limit","Limit the number of requests to a specific URL (except the root '/'). This can help in combating DDoS attacks on specific pages.","The number of requests that can be made in a certain amount of time.","The time frame in which the requests are counted (in seconds).")
	WRITE_XML_LIST(URL,ExcludedURL,"Excluded_Urls","Ignore certain urls from scanning. Once the url is determined to be one of these urls (after OnReadRawData event), the request will not be scanned by the firewall.","Exclude these urls from being scanned. These urls are case sensitive and special characters need to be encoded as it would appear in the HTTP request line.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLMappedPath(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Mapped Path
	xml += "<Mapped_Path App='Separator'/>\r\n";
	WRITE_XML_RULE(Path,ParentPath,"Parent_Path","Deny parent path ('..') attempt in the mapped path.")
	WRITE_XML_RULE(Path,SpecialWhitespace,"Special_Whitespace","Do not allow carriage return, line feed, backspace and tabulator characters.")
	WRITE_XML_RULE(Path,Escaping,"Escaping","Deny encoding exploits in the mapped path by blocking '%'.")
	WRITE_XML_RULE(Path,Dot,"Dot_In_Path","Deny a dot in the path (except for the filename).")
	WRITE_XML_RULE(Path,MultipleColons,"Multiple_Colons","Deny more than 1 colon (':') in the path.")
	WRITE_XML_RULE_CHARACTERS(Path,Characters,"Characters","Deny the request if one or more of these characters are present in the mapped path.")
	WRITE_XML_RULE_LIST(Path,AllowPaths,"Allowed_Paths","Only allow mapped paths which start with a certain path.","These are the allowed paths the mapped path may start with.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLRequestedFile(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Requested File
	xml += "<Requested_File App='Separator'/>\r\n";
	WRITE_XML_BOOL(Filename,UseRawScan,"Use_Filename_Raw_Scan","Besides using the default scanning, also use the raw scanning capability to scan the requested file before the web server decodes the URL (with built-in decoding engine).")
	WRITE_XML_RULE_CHARACTERS(Filename,Characters,"Filename_Characters","Deny the request if the filename contains one of these characters.")
	WRITE_XML_RULE(Filename,DefaultDocument,"Default_Document","Deny default document requests. The client can only request a specific file, not a directory.")
	WRITE_XML_RULE_LIST(Filename,DenyFiles,"Denied_Files","Deny the filenames/CGI applications being accessed or run.","These are the filenames/CGI applications that are not allowed.")
	WRITE_XML_LIST(Filename,MonitorFiles,"Monitored_Files","Monitor access to files.","These are the filenames that will be monitored.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLRequestedExtension(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Requested Extension
	WRITE_XML_RULE_LIST(Extensions,AllowExtensions,"Allowed_Extensions","Only allow requests for files with certain extensions.","These are the allowed extensions for the requested files.")
	WRITE_XML_RULE_LIST(Extensions,DenyExtensions,"Denied_Extensions","Deny requests for files with certain extensions.","These are the denied extensions for the requested files.")
	WRITE_XML_RULE_CACHE(Extensions,RequestsLimit,"Extension_Requests_Limit","Limit the number of requests an IP address can make to certain file extensions.","The number of requests that can be made in a certain amount of time.","The time frame in which the requests are counted (in minutes).")
	WRITE_XML_CSTRINGLIST(Extensions,LimitExtensions,"Limit_Extensions","Limit the number of requests to these file extensions.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLHTTPHeaders(CHTTPFirewallSettings &d)
{
	CString xml("");
	//HTTP Headers
	xml += "<Headers App='Separator'/>\r\n";
	WRITE_XML_RULE(Headers,RFCCompliantHostHeader,"RFC_Compliant_Host_Header","Block the HTTP 1.1 request if it does not include a 'Host:' header (RFC compliant).")
	WRITE_XML_RULE_LIST(Headers,DenyHeaders,"Denied_Headers","Block the request if any of the headers are present.","These are the headers not allowed in requests to the web server.")

	WRITE_XML_RULE(Headers,SQLInjection,"Header_SQL_Injection","Do not allow SQL injection in the headers sent to the web server.")
	WRITE_XML_RULE(Headers,EncodingExploits,"Header_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the headers sent to the web server.")
	WRITE_XML_RULE(Headers,DirectoryTraversal,"Header_Directory_Traversal","Do not allow directory traversal (parent path) in the headers sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_RULE(Headers,HighBitShellcode,"Header_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_RULE_INT(Headers,MaxLength,"Maximum_Header_Length","The maximum length of a single header.")
	WRITE_XML_RULE_MAP(Headers,MaxHeaders,"Max_Headers","Limit the length of additional headers. You can specify a custom header and a maximum length for each header in 'Max Headers'.","These are the headers of which the length is limited to a specific value.")
	WRITE_XML_RULE_LIST(Headers,DenySequences,"Denied_Header_Sequences","Block the request if any of the character sequences are present in the headers.","These are the character sequences not allowed in the headers.")
	WRITE_XML_RULE_MAP(Headers,DenyRegex,"Denied_Header_Regular_Expressions","Block the request if the headers match a certain regular expression.","The regular expressions to block. 'Key' is the name of the rule (will be logged). 'Value' is the regex pattern used to find matches (CAtlRegExp syntax).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLContentType(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Content-Type
	xml += "<Content_Type App='Separator'/>\r\n";
	WRITE_XML_RULE_LIST(ContentType,AllowContentTypes,"Allowed_Content_Types","Enable checking the Content-Type header in requests and deny the request if the Content-Type is not in the list.","These are the allowed Content-Types of requests. If for instance you want to enable all multipart types simply add 'multipart/'. This way you effectively enable 'multipart/form-data', 'multipart/mixed',... A blank line means no Content-Type is also allowed.")
	WRITE_XML_RULE_LIST(ContentType,DenyContentTypes,"Denied_Content_Types","Enable checking the Content-Type header in requests and deny the request if the Content-Type is in the list.","These are the denied Content-Types of requests. Examples are 'application/' (will block all application content-types), 'application/octet-stream', 'application/*' , ...")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLQuerystringScanning(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Querystring Scanning
	xml += "<Querystring App='Separator'/>\r\n";
	WRITE_XML_BOOL(QueryString,UseRawScan,"Use_Querystring_Raw_Scan","Besides using the default scanning, also use the raw scanning capability to scan the querystring before the web server decodes the URL (with built-in decoding engine).")
	WRITE_XML_RULE(QueryString,SQLInjection,"Querystring_SQL_Injection","Do not allow SQL injection in the querystring.")
	WRITE_XML_RULE(QueryString,EncodingExploits,"Querystring_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the querystring.")
	WRITE_XML_RULE(QueryString,DirectoryTraversal,"Querystring_Directory_Traversal","Do not allow directory traversal in the querystring. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_RULE(QueryString,HighBitShellcode,"Querystring_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_RULE(QueryString,SpecialWhitespace,"Querystring_Special_Whitespace","Do not allow carriage return, line feed, backspace and tabulator characters.")
	WRITE_XML_RULE_INT(QueryString,MaxLength,"Maximum_Querystring","Limit the length of the querystring (everything after the '?' in a url).")
	WRITE_XML_RULE_INT(QueryString,MaxVariableLength,"Maximum_Querystring_Variable_Length","The maximum length of a variable in the querystring.")
	WRITE_XML_RULE_LIST(QueryString,DenySequences,"Denied_Querystring_Sequences","Block the request if any of the sequences are present.","These are the character sequences not allowed in the querystring.")
	WRITE_XML_RULE_MAP(QueryString,DenyRegex,"Denied_Querystring_Regular_Expressions","Block the request if the querystring matches a certain regular expression.","The regular expressions to block. 'Key' is the name of the rule (will be logged). 'Value' is the regex pattern used to find matches (CAtlRegExp syntax).")
	WRITE_XML_LIST(QueryString,ExcludedQueryString,"Excluded_Querystrings","Ignore certain querystrings from scanning. Once the querystring is determined to be one of these querystrings (after OnReadRawData event), the request will not be scanned by the firewall. CAUTION: if attackers know these querystrings they can avoid detection!","Exclude these querystrings from being scanned. These are full querystrings, case sensitive and special characters need to be encoded as it would appear in the HTTP request line.")

	return xml;
}

CString CHTTPFirewallSettings::GetXMLGlobalFilterCapabilities(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Global Filter Capabilities
	xml += "<Global_Filter_Capabilities App='Separator'/>\r\n";
	WRITE_XML_BOOL(Global,IsInstalledAsGlobalFilter,"Is_Installed_As_Global_Filter","Register for the OnReadRawData event (required for the global filter capabilities). This event can only be called if the filter is installed as a global filter. If this is not the case then the filter will fail to load. For IIS 6, you need to run in IIS 5.0 Isolation mode. Use ISAPI extension instead for IIS6 (Worker Process mode) and IIS7+. Requires restart of IIS!")
	WRITE_XML_BOOL(Global,IsInstalledInWebProxy,"Is_Installed_In_Web_Proxy","Is the firewall installed in ISA Server or Forefront TMG. Requires restart!")
	WRITE_XML_RULE(Global,SlowHeaderAttack,"Slow_Header_Attack","Do not allow slow header attack. This is a DoS where headers are sent separately with long time intervals. This is only supported in ISAPI filter.")
	WRITE_XML_RULE(Global,SlowPostAttack,"Slow_Post_Attack","Do not allow slow POST attack. This is a DoS where postdata is sent in small packets with long time intervals. This is only supported in ISAPI filter.")
	WRITE_XML_RULE(Global,PostSQLInjection,"Postdata_SQL_Injection","Do not allow SQL injection in the data (e.g. postdata) sent to the web server.")
	WRITE_XML_RULE(Global,PostEncodingExploits,"Postdata_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in the data (e.g. postdata) sent to the web server.")
	WRITE_XML_RULE(Global,PostDirectoryTraversal,"Postdata_Directory_Traversal","Do not allow directory traversal (parent path) in the data (e.g. postdata) sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_RULE(Global,PostHighBitShellcode,"Postdata_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_RULE_INT(Global,MaxPostVariableLength,"Maximum_Postdata_Variable_Length","The maximum length of a variable in the data.")
	WRITE_XML_RULE_LIST(Global,DenyPostSequences,"Denied_Post_Sequences","Block the request if any of the character sequences are present in the data.","These are the character sequences not allowed in data (e.g. postdata).")
	WRITE_XML_RULE_MAP(Global,DenyPostRegex,"Denied_Post_Regular_Expressions","Block the request if the postdata match a certain regular expression.","The regular expressions to block. 'Key' is the name of the rule (will be logged). 'Value' is the regex pattern used to find matches (CAtlRegExp syntax).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLResponseMonitor(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Response Monitor
	xml += "<Response_Monitor App='Separator'/>\r\n";
	//WRITE_XML_BOOL(ResponseMonitor,RemoveServerHeader,"Remove_Server_Header","Remove the 'Server:' header in every response from the web server to the client. The server header is sensitive information that can be used by hackers or worms to probe for vulnerable systems or to know what they're up against.")
	//WRITE_XML_BOOL(ResponseMonitor,ChangeServerHeader,"Change_Server_Header","Instead of removing the server header, you can change it. This way you can fool hackers and worms by specifying another commercial web server in 'Server Header'. Note: 'Remove Server Header' has priority over this setting, so don't enable 'Remove Server Header' if you want to change the server header.")
	//WRITE_XML_STRING(ResponseMonitor,ServerHeader,"Server_Header","The server header sent back to the client in every response. This has to be in the form '<software>/<version>' (like 'Apache/0.8.4').")
	WRITE_XML_MAP(ResponseMonitor,Headers,"Response_Headers","Add, change or remove headers from the HTTP response","These are the headers that will be added or replaced in the response. Use an empty value to remove the header.")
	WRITE_XML_BOOL(ResponseMonitor,LogClientErrors,"Log_HTTP_Client_Errors","Log HTTP client side errors like '404 Not Found'. These errors start with a '4'.")
	WRITE_XML_RULE_CACHE(ResponseMonitor,ClientErrors,"HTTP_Client_Errors","Block the IP address if it generates too many HTTP client errors.","The maximum number of HTTP client errors that can be made in a certain amount of time.","The time frame in which the errors are counted (in minutes).")
	WRITE_XML_BOOL(ResponseMonitor,LogServerErrors,"Log_HTTP_Server_Errors","Log HTTP server side errors like '501 Not Implemented'. These errors start with a '5'.")
	WRITE_XML_RULE_CACHE(ResponseMonitor,ServerErrors,"HTTP_Server_Errors","Block the IP address if it generates too many HTTP server errors.","The maximum number of HTTP server errors that can be made in a certain amount of time.","The time frame in which the errors are counted (in minutes).")
	WRITE_XML_RULE_MAP(ResponseMonitor,InformationDisclosure,"Information_Disclosure","Deny certain information disclosures in text sent from the webserver to the client.","These are the information disclosures you want to block. 'Key' is the name of the rule (will be logged). 'Value' is the regex pattern used to find matches (CAtlRegExp syntax).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLSQLInjection(CHTTPFirewallSettings &d)
{
	CString xml("");
	//SQL Injection Keywords
	xml += "<SQL_Injection App='Separator'/>\r\n";
	xml += ConvertListToXML("SQL_Injection_Keywords",SQLi.Keywords,"These are the SQL keywords for the SQL injection scanning.");
	WRITE_XML_INT(SQLi,AllowedCount,"SQL_Injection_Allowed_Count","Ignore this number of matches. Only trigger an alert when more keywords than this maximum are found.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLEncodingExploits(CHTTPFirewallSettings &d)
{
	CString xml("");
	//Encoding Exploits
	xml += "<Encoding_Exploits App='Separator'/>\r\n";
	xml += ConvertListToXML("Encoding_Keywords",EncodingExploits.Keywords,"These are the keywords for detecting encoding exploits.");
	WRITE_XML_BOOL(EncodingExploits,ScanDoubleEncoding,"Detect_Double_Encoding","Scan for double encoding (= embedded encoding).")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLHTTPVerbs(CHTTPFirewallSettings &d)
{
	CString xml("");
	//HTTP Verbs (Request Methods)
	xml += "<Methods App='Separator'/>\r\n";
	WRITE_XML_RULE_LIST(Verbs,AllowVerbs,"Allowed_Verbs","Only allow certain request methods (HTTP verbs).","These are the request methods (HTTP verbs) that are allowed.")
	WRITE_XML_RULE_LIST(Verbs,DenyVerbs,"Denied_Verbs","Deny certain request methods (HTTP verbs).","These are the denied request methods (HTTP verbs).")
	return xml;
}

void CHTTPFirewallSettings::LoadDefaultsSQLInjection(CStringList& List)
{
	//SQL Injection (words in list should be lowercase, if 2 or more are found, request will be blocked)
	//used resources: next generation security software documents, Snort rules
	List.RemoveAll();
	List.AddTail("'");	//don't allow escaping string
	List.AddTail("`");	//MySQL string escape character
	List.AddTail(";");	//don't allow escaping numeric value
	List.AddTail("--");	//don't allow commenting
	List.AddTail("/*");	//don't allow commenting
	List.AddTail("*/");	//don't allow commenting
	List.AddTail("select");
	List.AddTail("insert");
	List.AddTail("update");
	List.AddTail("delete");
	List.AddTail("drop ");
	List.AddTail("alter ");	//false positive: "aalter"
	List.AddTail("create ");
	List.AddTail("backup ");
	List.AddTail("inner join");
	List.AddTail("from ");
	List.AddTail("where ");
	List.AddTail("union ");
	List.AddTail("group by");
	List.AddTail("having ");
	List.AddTail("table ");
	List.AddTail("shutdown");
	List.AddTail("kill ");
	List.AddTail("declare");
	List.AddTail("openrowset");
	List.AddTail("opendatasource");
	List.AddTail("pwdencrypt");
	List.AddTail("msdasql");
	List.AddTail("sqloledb");

	List.AddTail("quotename");
	List.AddTail("isnull");
	List.AddTail("xtype");
	List.AddTail("varchar");
	List.AddTail("char(");
	List.AddTail("char(124)");
	List.AddTail("char(9)");
	List.AddTail("char(94)");
	List.AddTail("char(32)");
	List.AddTail("char(85)");
	List.AddTail("char(60)"); //stored XSS <;
	List.AddTail("char(62)"); //stored XSS >;
	List.AddTail("cast(");

	List.AddTail("cursor");
	List.AddTail("fetch next");
	List.AddTail("allocate");	//also deallocate

	List.AddTail("raiserror");
	//List.AddTail("execute");  //detected by "exec"
	List.AddTail("exec");
	List.AddTail("=!(");
	List.AddTail("= !(");

	List.AddTail("dbo.");
	List.AddTail("master..");
	List.AddTail("@@version");
	List.AddTail("@@servername");
	List.AddTail("@@servicename");
	List.AddTail("@@fetch_status");

	List.AddTail("syslogins");
	List.AddTail("sysxlogins");
	List.AddTail("sysdatabases");
	List.AddTail("sysobjects");
	List.AddTail("syscomments");
	List.AddTail("syscolumns");
	List.AddTail("sysname");
	List.AddTail("system_user");

	List.AddTail("db_name");
	List.AddTail("db_id");
	List.AddTail("information_schema");
	List.AddTail("is_member");
	List.AddTail("is_srvrolemember");
	List.AddTail("object_id");
	List.AddTail("object_name");
	List.AddTail("col_length");
	List.AddTail("col_name");

	List.AddTail("0x31303235343830303536"); //Havij injection

	//see e-mail of Frank Block
	List.AddTail("' or '");
	List.AddTail("'or '");
	List.AddTail("'or'");
	List.AddTail("' or'");
	List.AddTail("or 1 --");
	List.AddTail("or 1--");
	List.AddTail("or 1=");

	/*
		Web Application Firewalls
		Evaluation and Analysis
		Andreas Karakannas
		George Thessalonikefs
	*/
	List.AddTail("||");
	List.AddTail("&&");

	List.AddTail("xp_");
	List.AddTail("sp_");
	//these below will be blocked (because the 2 lines above "xp_" & "sp_" will also be detected)
	List.AddTail("xp_cmdshell");
	List.AddTail("xp_reg");
	//List.AddTail("xp_regread");
	//List.AddTail("xp_regwrite");
	//List.AddTail("xp_regdeletekey");
	//List.AddTail("xp_regdeletevalue");
	//List.AddTail("xp_regaddmultistring");
	//List.AddTail("xp_regenumkeys");
	//List.AddTail("xp_regenumvalues");
	//List.AddTail("xp_regremovemultistring");
	List.AddTail("xp_servicecontrol");
	List.AddTail("xp_setsqlsecurity");
	List.AddTail("xp_readerrorlog");
	List.AddTail("xp_controlqueueservice"); //buffer overflow extended procedures
	List.AddTail("xp_createprivatequeue");
	List.AddTail("xp_decodequeuecommand");
	List.AddTail("xp_deleteprivatequeue");
	List.AddTail("xp_deletequeue");
	List.AddTail("xp_displayqueuemesgs");
	List.AddTail("xp_dsinfo");
	List.AddTail("xp_mergelineages");
	List.AddTail("xp_readpkfromqueue");
	List.AddTail("xp_readpkfromvarbin");
	List.AddTail("xp_repl_encrypt");
	List.AddTail("xp_resetqueue");
	List.AddTail("xp_sqlinventory");
	List.AddTail("xp_unpackcab");
	List.AddTail("xp_sprintf");
	List.AddTail("xp_displayparamstmt");
	List.AddTail("xp_enumresult");
	List.AddTail("xp_showcolv");
	List.AddTail("xp_updatecolvbm");
	List.AddTail("xp_execresultset"); //can be used to bypass access control
	List.AddTail("xp_printstatements");
	List.AddTail("xp_peekqueue");
	List.AddTail("xp_proxiedmetadata");
	List.AddTail("xp_displayparamstmt");
	List.AddTail("xp_availablemedia");
	List.AddTail("xp_enumdsn");
	List.AddTail("xp_filelist");
	List.AddTail("sp_password");
	List.AddTail("sp_adduser");
	List.AddTail("sp_addextendedproc");
	List.AddTail("sp_dropextendedproc");
	List.AddTail("sp_add_job"); //sp_add_jobstep
	List.AddTail("sp_start_job");
	List.AddTail("sp_delete_alert");
	List.AddTail("sp_msrepl_startup");
	List.AddTail("sp_configure"); //SQL Server 2005
}

void CHTTPFirewallSettings::LoadDefaultsEncodingExploits(CStringList& List)
{
	List.RemoveAll();
	//hex null
	List.AddTail("%00");

	//unicode
	List.AddTail("%u");
	List.AddTail("%u00");
	List.AddTail("%u0000");
	List.AddTail("%U");
	List.AddTail("%U0000");
	List.AddTail("%U00000000");

	//C-style
	List.AddTail("\\o00");
	List.AddTail("\\x00");
	List.AddTail("\\u0000");
	List.AddTail("\\U00000000");
}

void CHTTPFirewallSettings::LoadDefaultsAuthPasswords(CStringList& List)
{
	//AUTH - Default passwords
	List.RemoveAll();
	List.AddTail("1111");
	List.AddTail("11111");
	List.AddTail("111111");
	List.AddTail("1234");
	List.AddTail("12345");
	List.AddTail("123456");
	List.AddTail("1234567");
	List.AddTail("12345678");
	List.AddTail("123456789");
	List.AddTail("1234567890");
	List.AddTail("54321");
	List.AddTail("adm");
	List.AddTail("admin");
	List.AddTail("administrator");
	List.AddTail("access");
	List.AddTail("account");
	List.AddTail("all");
	List.AddTail("anyone");
	List.AddTail("azerty");
	List.AddTail("backup");
	List.AddTail("control");
	List.AddTail("database");
	List.AddTail("dba");
	List.AddTail("debug");
	List.AddTail("default");
	List.AddTail("develop");
	List.AddTail("developer");
	List.AddTail("development");
	List.AddTail("enter");
	List.AddTail("ftp");
	List.AddTail("ftproot");
	List.AddTail("god");
	List.AddTail("guest");
	List.AddTail("login");
	List.AddTail("master");
	List.AddTail("oracle");
	List.AddTail("oracle8");
	List.AddTail("password");
	List.AddTail("password1");
	List.AddTail("pass");
	List.AddTail("pwrchute");
	List.AddTail("qwerty");
	List.AddTail("root");
	List.AddTail("sa");
	List.AddTail("secret");
	List.AddTail("sql");
	List.AddTail("sqlserver");
	List.AddTail("tech");
	List.AddTail("test");
	List.AddTail("user");
	List.AddTail("web");
	List.AddTail("webmaster");
	List.AddTail("work");
	List.AddTail("www");
}

void CHTTPFirewallSettings::LoadDefaultsAuthAccounts(CStringList& List)
{
	List.RemoveAll();
	List.AddTail("Guest");
	List.AddTail("SQLDebugger");
	List.AddTail("TsInetnetUser");
	List.AddTail("Test");
	List.AddTail("SQL");
	List.AddTail("SQLServer");
	List.AddTail("Webserver");
	List.AddTail("nobody");
	List.AddTail("postmaster");
	List.AddTail("operator");
	List.AddTail("Mail");
	List.AddTail("Replicator");
	List.AddTail("Replication");
	List.AddTail("Debug");
	List.AddTail("DBA");
	List.AddTail("abc");
	List.AddTail("access");
	List.AddTail("admin");
	//List.AddTail("administrator");
	List.AddTail("anon");
	List.AddTail("anonymous");
	List.AddTail("anyone");
	List.AddTail("backup");
	List.AddTail("batch");
	List.AddTail("config");
	List.AddTail("control");
	List.AddTail("data");
	List.AddTail("database");
	List.AddTail("ftp");
	List.AddTail("ftproot");
	List.AddTail("install");
	List.AddTail("login");
	List.AddTail("master");
	List.AddTail("oracle");
	List.AddTail("oracle8");
	List.AddTail("public");
	List.AddTail("pwrchute");
	List.AddTail("recovery");
	List.AddTail("remote");
	List.AddTail("root");
	List.AddTail("router");
	List.AddTail("sa");
	List.AddTail("security");
	List.AddTail("server");
	List.AddTail("service");
	List.AddTail("setup");
	List.AddTail("sybase");
	List.AddTail("test");
	List.AddTail("user");
	List.AddTail("web");
	List.AddTail("webdb");
	List.AddTail("webmaster");
	List.AddTail("www");
	List.AddTail("wwwadmin");
}

void CHTTPFirewallSettings::LoadDefaultsUnixCommands(CStringList& List)
{
	//Default *nix commands
	//List.AddTail("ps");
	List.AddTail("wget"); //too much collateral damage
	List.AddTail("uname");
	//List.AddTail("id");
	List.AddTail("echo");
	List.AddTail("kill");
	List.AddTail("chmod");
	List.AddTail("chgrp");
	List.AddTail("chsh");
	//List.AddTail("cc");
	//List.AddTail("cpp");
	List.AddTail("gcc");
	List.AddTail("g++");
	List.AddTail("python");	//too much collateral damage
	List.AddTail("tclsh");
	List.AddTail("nasm");
	//List.AddTail("perl");	//too much collateral damage: hyperlink.htm
	List.AddTail("traceroute");
	List.AddTail("nmap");	//too much collateral damage
	List.AddTail("lsof");	
	//List.AddTail("mail");
	List.AddTail("inetd.conf");
	List.AddTail("motd");
	List.AddTail("shadow");
	List.AddTail("httpd.conf"); //*/
}

void CHTTPFirewallSettings::LoadDefaultsHTTPVersions(CStringList& List)
{
	List.RemoveAll();
	List.AddTail("");			//HTTP/0.9
	List.AddTail("HTTP/0.9");	//IIS 6 reports empty version as "HTTP/0.9"
	List.AddTail("HTTP/1.0");
	List.AddTail("HTTP/1.1");
	List.AddTail("HTTP/2.0");
}

CString CHTTPFirewallSettings::GetXMLLogging(CHTTPFirewallSettings &d)
{
	CString xml("");
	xml += "<Logging App='Separator'/>\r\n";
	xml += CFirewallSettings::GetXMLLogging(d);
	WRITE_XML_BOOL(HTTPLogging,Log_HTTP_VIA,"Log_HTTP_VIA","Log the 'Via:' header to have a clue where the original request came from (if the client uses 1 or more proxies). Note: you will not be able to log all used proxies (certain proxies don't have or remove this header)!")
	WRITE_XML_BOOL(HTTPLogging,Log_HTTP_X_FORWARDED_FOR,"Log_HTTP_X_FORWARDED_FOR","Log the 'X_Forwarded_For:' header. Certain proxies (like NetCache) add this header to the request which indicate the source IP address of the request.")
	WRITE_XML_BOOL(HTTPLogging,LogHostHeader,"Log_Host_Header","Log the host header. This will log the host header of the request, so you will have a clue to what web site the request was intended.")
	WRITE_XML_BOOL(HTTPLogging,LogUserAgent,"Log_User_Agent","Log the client user agent. This can indicate what software/tool is used to perform the attack. However it is not essential information for reporting an abuse.")
	return xml;
}

CString CHTTPFirewallSettings::ToXML()
{
	CString xml;
	CHTTPFirewallSettings d;
	xml += GetXMLAdmin(d);
	xml += GetXMLScanningEngine(d);
	xml += GetXMLResponseHandling(d);
	xml += GetXMLLogging(d);
	xml += GetXMLConnectionControl(d);
	xml += GetXMLAuthentication(d);
	xml += GetXMLRequestLimits(d);
	xml += GetXMLURLScanning(d);
	xml += GetXMLMappedPath(d);
	xml += GetXMLRequestedFile(d);
	xml += GetXMLRequestedExtension(d);
	xml += GetXMLHTTPHeaders(d);
	xml += GetXMLContentType(d);
	xml += GetXMLCookie(d);
	xml += GetXMLUserAgent(d);
	xml += GetXMLReferrer(d);
	xml += GetXMLHTTPVerbs(d);
	xml += GetXMLQuerystringScanning(d);
	xml += GetXMLGlobalFilterCapabilities(d);
	xml += GetXMLResponseMonitor(d);
	xml += GetXMLSQLInjection(d);
	return xml;
}

bool CHTTPFirewallSettings::ReadFromFile(LPCTSTR fn)
{
	bool ret(false);

	//read settings file
	ret = CSettings::ReadFromFile(fn);
	//load agents settings
	LoadAgents();

	return ret;
}

void CHTTPFirewallSettings::ReadAgentsFile(LPCTSTR Path)
{
	//load database
	if(Agents.AgentList.ReadFromFile(Path)){
		LoadAgents();
	}else{
		/*
		//if database does not exist yet, make one with defaults
		if (!FileExists(Path)){
			Agents.AgentList.WriteToFile(Path);
		}//*/
	}
	//Agents.AgentList.WriteToFile("C:\\Agents.xml");
}

CString CHTTPFirewallSettings::GetXMLCookie(CHTTPFirewallSettings &d)
{
	//Cookie
	CString xml("");
	xml += "<Cookie App='Separator'/>\r\n";
	WRITE_XML_RULE(Cookie,SQLInjection,"Cookie_SQL_Injection","Deny SQL injection in the 'Cookie:' header. This can be useful if your website is using a database and you are using cookies for storing information related to the database.")
	WRITE_XML_RULE(Cookie,EncodingExploits,"Cookie_Encoding_Exploits","Do not allow encoding exploits (embedded encoding) in cookies (in the 'Cookie:' header).")
	WRITE_XML_RULE(Cookie,DirectoryTraversal,"Cookie_Directory_Traversal","Do not allow directory traversal (parent path) in the cookie sent to the web server. This will block any '..' preceding or following a slash ('/' or '\\').")
	WRITE_XML_RULE(Cookie,HighBitShellcode,"Cookie_High_Bit_Shellcode","Do not allow high bit shellcode (ascii>127). This will restrict the web sites to US-ASCII only and block characters not in this character set. Not recommended for non-US-English web sites.")
	WRITE_XML_RULE(Cookie,SpecialWhitespace,"Cookie_Special_Whitespace","Do not allow carriage return, line feed, backspace and tabulator characters.")
	WRITE_XML_RULE_INT(Cookie,MaxVariableLength,"Maximum_Cookie_Variable_Length","The maximum length of a variable in the cookies.")
	WRITE_XML_RULE_LIST(Cookie,DenySequences,"Denied_Cookie_Sequences","Block the request if any of the character sequences are present in the cookie.","These are the character sequences not allowed in the cookie.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLUserAgent(CHTTPFirewallSettings &d)
{
	//User-Agent
	CString xml("");
	xml += "<User_Agent App='Separator'/>\r\n";
	WRITE_XML_RULE(UserAgent,Empty,"User_Agent_Empty","Deny the request if the user agent is empty or not present.")
	WRITE_XML_RULE(UserAgent,NonRFC,"User_Agent_Non_RFC","Deny the request if the user agent is not RFC compliant.")
	WRITE_XML_RULE(UserAgent,HighBitShellcode,"User_Agent_High_Bit_Shellcode","Deny high bit shell code in the user agent. This will block ASCII>127 and possibly blocking non US-ASCII web browsers user agent strings.")
	WRITE_XML_RULE(UserAgent,SpecialWhitespace,"User_Agent_Special_Whitespace","Do not allow carriage return, line feed, backspace and tabulator characters.")
	WRITE_XML_RULE_CHARACTERS(UserAgent,RequireCharacter,"Require_User_Agent_Character","Deny the request if the user agent does not contain at least one of these characters.")
	WRITE_XML_RULE_CACHE(UserAgent,Switching,"User_Agent_Switching","Deny the request if the user agent is changing too much coming from a single IP address.","The maximum number of different user agents in a certain amount of time.","The time frame in which the user agents are counted (in minutes).")
	WRITE_XML_RULE_LIST(UserAgent,DenyUserAgents,"Denied_User_Agents","Deny the request for certain user agent strings.","These are the denied user agents of requests.")
	WRITE_XML_RULE_LIST(UserAgent,DenySequences,"Denied_User_Agent_Sequences","Deny the request if the user agent contains a certain character sequence.","These are the denied user agents sequences.")
	WRITE_XML_LIST(UserAgent,Excluded,"Excluded_User_Agents","Exclude requests with certain user agent strings.","Exclude the request from being scanned when the User-Agent header is one of these.")
	return xml;
}

CString CHTTPFirewallSettings::GetXMLReferrer(CHTTPFirewallSettings &d)
{
	//Referrer
	CString xml("");
	xml += "<Referrer App='Separator'/>\r\n";
	WRITE_XML_BOOL(Referrer,UseScan,"Use_Referrer_Scanning","Scan the referrer URL. Enabling this allows the other checks in this section.")
	WRITE_XML_RULE(Referrer,RFCCompliantURL,"Referrer_URL_RFC_Compliant","The referrer URL has to be RFC compliant.")
	WRITE_XML_RULE(Referrer,RFCCompliantHTTPURL,"Referrer_URL_RFC_HTTPCompliant","The referrer URL has to be HTTP RFC compliant (no authentication).")
	WRITE_XML_RULE(Referrer,EncodingExploits,"Referrer_Encoding_Exploits","Deny encoding exploits and embedded encoding in the referrer URL.")
	WRITE_XML_RULE(Referrer,HotLinking,"Referrer_Hot_Linking","Deny hot linking (also called direct linking, inline linking) to certain urls or file extensions from certain domains.")
	WRITE_XML_CSTRINGLIST(Referrer,HotLinkingUrls,"Referrer_Hot_Linking_Urls","The urls hot linking to is denied. This can also be used to prevent cross-site request forgery (CSRF) on those urls if Deny Blank Referrer is enabled.")
	WRITE_XML_CSTRINGLIST(Referrer,HotLinkingFileExtensions,"Referrer_Hot_Linking_File_Extensions","The file extensions hot linking to is denied.")
	WRITE_XML_RULE_LIST(Referrer,HotLinkingAllowDomains,"Referrer_Hot_Linking_Allow_Domains","Only allow certain domains to use hot linking.","The domains (FQDN) or IP addresses that are allowed to use hot linking. You do not need to add your own domain to this list, see setting: \"Use Host Header\".")
	WRITE_XML_RULE_LIST(Referrer,HotLinkingDenyDomains,"Referrer_Hot_Linking_Deny_Domains","Deny certain domains to use hot linking.","The domains (FQDN) or IP addresses that are denied to use hot linking.")
	WRITE_XML_BOOL(Referrer,HotLinkingUseHostHeader,"Referrer_Hot_Linking_Use_Host_Header","Allow the Host header domain to use hot linking. This is allowing the local web site to refer to itself without needing to add the domain names to the allowed list above.")
	WRITE_XML_RULE(Referrer,HotLinkingBlankReferrer,"Referrer_Hot_Linking_Blank_Referrer","Deny requests with no referrer to the protected file extensions or urls. This will block some leeching tools but also some proxy servers and browsers with additional security applications that remove the referrer header.")
	WRITE_XML_RULE_CHARACTERS(Referrer,Characters,"Referrer_Characters","Deny certain characters in the referrer URL.")
	WRITE_XML_RULE(Referrer,HighBitShellcode,"Referrer_High_Bit_Shellcode","Deny high bit shell code in the referrer URL. This will block ASCII>127 in the referrer URL and possibly blocking non US-ASCII web sites from linking to your site.")
	WRITE_XML_RULE(Referrer,SpecialWhitespace,"Referrer_Special_Whitespace","Do not allow carriage return, line feed, backspace and tabulator characters.")
	WRITE_XML_RULE(Referrer,SQLInjection,"Referrer_SQL_Injection","Do not allow SQL injection in the referrer URL sent to the web server.")
	WRITE_XML_RULE_LIST(Referrer,DenySequences,"Deny_Referrer_Sequences","Deny certain character sequences in the referrer URL.","The list of denied character sequences in the referrer URL.")
	return xml;
}

bool CHTTPFirewallSettings::Update(bool force)
{
	if(Agents.AgentList.Update(force)){
		//reload main settings after agents
		if(UpdateStamp.FileName!=""){
			ReadFromFile(UpdateStamp.FileName);
			return true;
		}else{
			return false; //no filename like when defaults loaded
		}
	}else{
		return CFirewallSettings::Update(force); //reload agents is actually done in ReadFromFile()
	}
}

void CHTTPFirewallSettings::AddJavaScriptKeywords(CStringList& list)
{
	list.AddTail("javascript:");
	list.AddTail("behavior:");
	list.AddTail("onabort");
	list.AddTail("onactivate");		//added 2011
	list.AddTail("onafterprint");		//added 2011
	list.AddTail("onafterupdate");		//added 2011
	list.AddTail("onbeforeactivate");	//added 2011
	list.AddTail("onbeforecopy");		//added 2011
	list.AddTail("onbeforecut");		//added 2011
	list.AddTail("onbeforepaste");		//added 2011
	list.AddTail("onbeforedeactivate");//added 2011
	list.AddTail("onbeforeeditfocus");	//added 2011
	list.AddTail("onbeforeunload");	//added 2011/Fixed 2013 o->u
	list.AddTail("onbeforeprint");		//added 2011
	list.AddTail("onbeforeupdate");	//added 2011
	list.AddTail("onblur");
	list.AddTail("onbounce");			//added 2013 (marquee) Reported by Rafay Baloch
	list.AddTail("oncanplay");			//added 2011
	list.AddTail("oncanplaythrough");	//added 2011
	list.AddTail("oncellchange");		//added 2011
	list.AddTail("onchange");
	list.AddTail("onclick");
	list.AddTail("oncontextmenu");		//added 2011
	list.AddTail("oncontrolselect");	//added 2011
	list.AddTail("oncopy");			//added 2011
	list.AddTail("oncut");				//added 2011
	list.AddTail("ondataavailable");	//added 2011
	list.AddTail("ondatasetchanged");	//added 2011
	list.AddTail("ondatasetcomplete");	//added 2011
	list.AddTail("ondblclick");
	list.AddTail("ondeactivate");		//added 2011
	list.AddTail("ondragdrop");
	list.AddTail("ondrag");			//added 2011
	list.AddTail("ondragend");			//added 2011
	list.AddTail("ondragenter");		//added 2011
	list.AddTail("ondragleave");		//added 2011
	list.AddTail("ondragover");		//added 2011
	list.AddTail("ondragstart");		//added 2011
	list.AddTail("ondrop");			//added 2011
	list.AddTail("ondurationchange");	//added 2011
	list.AddTail("onemptied");			//added 2011
	list.AddTail("onerror");
	list.AddTail("onended");			//added 2011
	list.AddTail("onerrorupdate");		//added 2011
	list.AddTail("onfilterchange");	//added 2011
	list.AddTail("onfinish");			//added 2013 (marquee) Reported by Rafay Baloch
	list.AddTail("onfocus");
	list.AddTail("onfocusin");			//added 2011
	list.AddTail("onfocusout");		//added 2011
	list.AddTail("onformchange");		//added 2011
	list.AddTail("onforminput");		//added 2011
	list.AddTail("onhaschange");		//added 2011
	list.AddTail("onhashchange");		//added 2013 http://help.dottoro.com/larrqqck.php
	list.AddTail("onhelp");			//added 2011
	list.AddTail("oninput");			//added 2011
	list.AddTail("oninvalid");			//added 2011
	list.AddTail("onkeydown");
	list.AddTail("onkeypress");
	list.AddTail("onkeyup");
	list.AddTail("onlayoutcomplete");	//added 2011                                    
	list.AddTail("onload");
	list.AddTail("onloadeddata");		//added 2011
	list.AddTail("onloadedmetadata");	//added 2011
	list.AddTail("onloadedstart");		//added 2011
	list.AddTail("onlosecapture");		//added 2011
	list.AddTail("onmessage");			//added 2011
	list.AddTail("onmousedown");
	list.AddTail("onmouseenter");		//added 2011
	list.AddTail("onmouseleave");		//added 2011
	list.AddTail("onmousemove");		//added 2011
	list.AddTail("onmouseout");
	list.AddTail("onmouseover");
	list.AddTail("onmouseup");
	list.AddTail("onmousewheel");		//added 2011
	list.AddTail("onmove");
	list.AddTail("onmoveend");			//added 2011
	list.AddTail("onmovestart");		//added 2011
	list.AddTail("onoffline");			//added 2011
	list.AddTail("ononline");			//added 2011
	list.AddTail("onpagehide");		//added 2011
	list.AddTail("onpageshow");		//added 2011
	list.AddTail("onpaste");			//added 2011
	list.AddTail("onpause");			//added 2011
	list.AddTail("onplay");			//added 2011
	list.AddTail("onplaying");			//added 2011
	list.AddTail("onpopstate");		//added 2011
	list.AddTail("onprogress");		//added 2011
	list.AddTail("onpropertychange");	//added 2011
	list.AddTail("onratechange");		//added 2011
	list.AddTail("onreadystatechange");//added 2011
	list.AddTail("onredo");			//added 2011
	list.AddTail("onreset");
	list.AddTail("onresize");
	list.AddTail("onresizeend");		//added 2011
	list.AddTail("onresizestart");		//added 2011
	list.AddTail("onrowenter");		//added 2011
	list.AddTail("onrowexit");			//added 2011
	list.AddTail("onrowsdelete");		//added 2011
	list.AddTail("onrowsinserted");	//added 2011
	list.AddTail("onscroll");			//added 2011
	list.AddTail("onsearch");			//added 2013 http://help.dottoro.com/larrqqck.php
	list.AddTail("onseeked");			//added 2011
	list.AddTail("onseeking");			//added 2011
	list.AddTail("onselect");
	list.AddTail("onselectionchange");	//added 2013 http://help.dottoro.com/larrqqck.php
	list.AddTail("onselectstart");		//added 2011
	list.AddTail("onshow");				//added 2015 Reported by Mazin Ahmed
	list.AddTail("onstalled");			//added 2011
	list.AddTail("onstart");			//added 2013 (marquee) Reported by Rafay Baloch
	list.AddTail("onstop");				//added 2013 http://help.dottoro.com/larrqqck.php
	list.AddTail("onstorage");			//added 2011
	list.AddTail("onsubmit");
	list.AddTail("onsuspend");			//added 2011
	list.AddTail("ontimeerror");		//added 2011
	list.AddTail("ontimeupdate");		//added 2011
	list.AddTail("ontoggle");			//added 2015 Reported by Mazin Ahmed
	list.AddTail("onundo");			//added 2011
	list.AddTail("onunload");
	list.AddTail("onvolumechange");	//added 2011
	list.AddTail("onwaiting");			//added 2011
	list.AddTail("style=");				//added 2011
	list.AddTail("<isindex");			//added 2013 (isindex) Reported by Rafay Baloch
	list.AddTail("formaction");			//added 2013 (isindex) Reported by Rafay Baloch
}

void CHTTPFirewallSettings::UpgradeJavaScriptKeywords(CStringList& list)
{
	//add keywords (Reported by Mazin Ahmed)
	AddNotInList(list,"onshow");
	AddNotInList(list,"ontoggle");

	//add keywords (Reported by Rafay Baloch)
	AddNotInList(list,"onbounce");
	AddNotInList(list,"onfinish");
	AddNotInList(list,"onstart");
	AddNotInList(list,"onhashchange");
	AddNotInList(list,"onsearch");
	AddNotInList(list,"onselectionchange");
	AddNotInList(list,"onstop");
	AddNotInList(list,"<isindex");
	AddNotInList(list,"formaction");

	//remove trailing =
	ReplaceKeyword(list,"onabort=","onabort");
	ReplaceKeyword(list,"onactivate=","onactivate");		
	ReplaceKeyword(list,"onafterprint=","onafterprint");		
	ReplaceKeyword(list,"onafterupdate=","onafterupdate");		
	ReplaceKeyword(list,"onbeforeactivate=","onbeforeactivate");	
	ReplaceKeyword(list,"onbeforecopy=","onbeforecopy");		
	ReplaceKeyword(list,"onbeforecut=","onbeforecut");		
	ReplaceKeyword(list,"onbeforepaste=","onbeforepaste");		
	ReplaceKeyword(list,"onbeforedeactivate=","onbeforedeactivate");
	ReplaceKeyword(list,"onbeforeeditfocus=","onbeforeeditfocus");	
	ReplaceKeyword(list,"onbeforeonload=","onbeforeunload");
	ReplaceKeyword(list,"onbeforeunload=","onbeforeunload");
	ReplaceKeyword(list,"onbeforeprint=","onbeforeprint");		
	ReplaceKeyword(list,"onbeforeupdate=","onbeforeupdate");	
	ReplaceKeyword(list,"onblur=","onblur");
	ReplaceKeyword(list,"oncanplay=","oncanplay");			
	ReplaceKeyword(list,"oncanplaythrough=","oncanplaythrough");	
	ReplaceKeyword(list,"oncellchange=","oncellchange");		
	ReplaceKeyword(list,"onchange=","onchange");
	ReplaceKeyword(list,"onclick=","onclick");
	ReplaceKeyword(list,"oncontextmenu=","oncontextmenu");		
	ReplaceKeyword(list,"oncontrolselect=","oncontrolselect");	
	ReplaceKeyword(list,"oncopy=","oncopy");			
	ReplaceKeyword(list,"oncut=","oncut");				
	ReplaceKeyword(list,"ondataavailable=","ondataavailable");	
	ReplaceKeyword(list,"ondatasetchanged=","ondatasetchanged");	
	ReplaceKeyword(list,"ondatasetcomplete=","ondatasetcomplete");	
	ReplaceKeyword(list,"ondblclick=","ondblclick");
	ReplaceKeyword(list,"ondeactivate=","ondeactivate");		
	ReplaceKeyword(list,"ondragdrop=","ondragdrop");
	ReplaceKeyword(list,"ondrag=","ondrag");			
	ReplaceKeyword(list,"ondragend=","ondragend");			
	ReplaceKeyword(list,"ondragenter=","ondragenter");		
	ReplaceKeyword(list,"ondragleave=","ondragleave");		
	ReplaceKeyword(list,"ondragover=","ondragover");		
	ReplaceKeyword(list,"ondragstart=","ondragstart");		
	ReplaceKeyword(list,"ondrop=","ondrop");			
	ReplaceKeyword(list,"ondurationchange=","ondurationchange");	
	ReplaceKeyword(list,"onemptied=","onemptied");			
	ReplaceKeyword(list,"onerror=","onerror");
	ReplaceKeyword(list,"onended=","onended");			
	ReplaceKeyword(list,"onerrorupdate=","onerrorupdate");		
	ReplaceKeyword(list,"onfilterchange=","onfilterchange");	
	ReplaceKeyword(list,"onfocus=","onfocus");
	ReplaceKeyword(list,"onfocusin=","onfocusin");			
	ReplaceKeyword(list,"onfocusout=","onfocusout");		
	ReplaceKeyword(list,"onformchange=","onformchange");		
	ReplaceKeyword(list,"onforminput=","onforminput");		
	ReplaceKeyword(list,"onhaschange=","onhaschange");		
	ReplaceKeyword(list,"onhelp=","onhelp");			
	ReplaceKeyword(list,"oninput=","oninput");			
	ReplaceKeyword(list,"oninvalid=","oninvalid");			
	ReplaceKeyword(list,"onkeydown=","onkeydown");
	ReplaceKeyword(list,"onkeypress=","onkeypress");
	ReplaceKeyword(list,"onkeyup=","onkeyup");
	ReplaceKeyword(list,"onlayoutcomplete=","onlayoutcomplete");	                                    
	ReplaceKeyword(list,"onload=","onload");
	ReplaceKeyword(list,"onloadeddata=","onloadeddata");		
	ReplaceKeyword(list,"onloadedmetadata=","onloadedmetadata");	
	ReplaceKeyword(list,"onloadedstart=","onloadedstart");		
	ReplaceKeyword(list,"onlosecapture=","onlosecapture");		
	ReplaceKeyword(list,"onmessage=","onmessage");			
	ReplaceKeyword(list,"onmousedown=","onmousedown");
	ReplaceKeyword(list,"onmouseenter=","onmouseenter");		
	ReplaceKeyword(list,"onmouseleave=","onmouseleave");		
	ReplaceKeyword(list,"onmousemove=","onmousemove");		
	ReplaceKeyword(list,"onmouseout=","onmouseout");
	ReplaceKeyword(list,"onmouseover=","onmouseover");
	ReplaceKeyword(list,"onmouseup=","onmouseup");
	ReplaceKeyword(list,"onmousewheel=","onmousewheel");		
	ReplaceKeyword(list,"onmove=","onmove");
	ReplaceKeyword(list,"onmoveend=","onmoveend");			
	ReplaceKeyword(list,"onmovestart=","onmovestart");		
	ReplaceKeyword(list,"onoffline=","onoffline");			
	ReplaceKeyword(list,"ononline=","ononline");			
	ReplaceKeyword(list,"onpagehide=","onpagehide");		
	ReplaceKeyword(list,"onpageshow=","onpageshow");		
	ReplaceKeyword(list,"onpaste=","onpaste");			
	ReplaceKeyword(list,"onpause=","onpause");			
	ReplaceKeyword(list,"onplay=","onplay");			
	ReplaceKeyword(list,"onplaying=","onplaying");			
	ReplaceKeyword(list,"onpopstate=","onpopstate");		
	ReplaceKeyword(list,"onprogress=","onprogress");		
	ReplaceKeyword(list,"onpropertychange=","onpropertychange");	
	ReplaceKeyword(list,"onratechange=","onratechange");		
	ReplaceKeyword(list,"onreadystatechange=","onreadystatechange");
	ReplaceKeyword(list,"onredo=","onredo");			
	ReplaceKeyword(list,"onreset=","onreset");
	ReplaceKeyword(list,"onresize=","onresize");
	ReplaceKeyword(list,"onresizeend=","onresizeend");		
	ReplaceKeyword(list,"onresizestart=","onresizestart");		
	ReplaceKeyword(list,"onrowenter=","onrowenter");		
	ReplaceKeyword(list,"onrowexit=","onrowexit");			
	ReplaceKeyword(list,"onrowsdelete=","onrowsdelete");		
	ReplaceKeyword(list,"onrowsinserted=","onrowsinserted");	
	ReplaceKeyword(list,"onscroll=","onscroll");			
	ReplaceKeyword(list,"onseeked=","onseeked");			
	ReplaceKeyword(list,"onseeking=","onseeking");			
	ReplaceKeyword(list,"onselect=","onselect");
	ReplaceKeyword(list,"onselectstart=","onselectstart");		
	ReplaceKeyword(list,"onstalled=","onstalled");			
	ReplaceKeyword(list,"onstorage=","onstorage");			
	ReplaceKeyword(list,"onsubmit=","onsubmit");
	ReplaceKeyword(list,"onsuspend=","onsuspend");			
	ReplaceKeyword(list,"ontimeerror=","ontimeerror");		
	ReplaceKeyword(list,"ontimeupdate=","ontimeupdate");		
	ReplaceKeyword(list,"onundo=","onundo");			
	ReplaceKeyword(list,"onunload=","onunload");
	ReplaceKeyword(list,"onvolumechange=","onvolumechange");
	ReplaceKeyword(list,"onwaiting=","onwaiting");
}